#!/usr/bin/env python
# Copyright 2016 Cumulus Networks, Inc
#
# ecmpcalc utility helpers

import subprocess
import cumulus.porttab
import re

########################################################################
# ecmpcalc portmap helpers
########################################################################
def modport2hw(mod, port, chip):
    if chip in ('Trident2', 'Trident2+'):
        hwport = port
    elif chip in ('Trident', 'Trident+'):
        hwport = (mod << 5) | port
    else:
        raise RuntimeError('ecmpcalc is not available on %s' % chip)
    return hwport

def hw2modport(hw, chip):
    if chip in ('Trident2', 'Trident2+', 'Spectrum'):
        mod = 0
        port = hw
    elif chip in ('Trident', 'Trident+'):
        mod = hw >> 5
        port = hw & 0x1f
    else:
        raise RuntimeError('ecmpcalc is not available on %s' % chip)
    return (mod, port)

def modport2interface(mod, port, chip):
    if chip in ('Trident3'):
        pt = cumulus.porttab.porttab()
        return pt.logical2dport(port)
    else:
        hw = modport2hw(mod, port, chip) - 1
        return 'xe%d' % hw

def interface2modport(interface, chip):
    inum = int(interface[2:])
    return hw2modport(inum + 1, chip)

def asic2linux(asic_port):
    pt = cumulus.porttab.porttab()
    return pt.bcm2linux(asic_port)

def asictrunkmbr2linux(asic_port):
    pt = asic2linux(asic_port)
    # locate master
    cmd = [ "/bin/ip", "link", "show", pt]
    try:
        linkDispStr = subprocess.check_output(cmd)
    except subprocess.CalledProcessError:
        return 'unk'
    pat=re.compile('master (?P<bond>\S+)')
    obj=pat.search(linkDispStr)
    return obj.group('bond') if obj else 'unk'

def linux2modport(linux, chip):
    pt = cumulus.porttab.porttab()
    if pt.logical_ports_available():
        mod = 0
        port = pt.linux2logical(linux)
    else:
        asic_port = pt.linux2bcm(linux)
        (mod, port) = interface2modport(asic_port, chip)
    return (mod, port)

