#
# Copyright 2016.  Cumulus Networks, Inc.
#
import os
import subprocess
import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors
from cumulus.platforms.cel_bigstone_g_common import *

class CelBigStoneG100Switch(cumulus.platform.BCMSwitch):
    _fab = cumulus.platform.Tomahawk100GFabricPort
    has_phy_ucode_mdio = False
    has_phy_ucode_spi_rom = False

    ports = (

        # Fabric Card Tomahawk to lc #1
        _fab(label="0",  serdes=( 68,  69,  70,  71),
              tx_polarity_flip=0x000f, rx_polarity_flip=0x0000,
              tx_lane_map=0x3210, rx_lane_map=0x0123),
        _fab(label="1",  serdes=( 72,  73,  74,  75),
              tx_polarity_flip=0x000f, rx_polarity_flip=0x0000,
              tx_lane_map=0x3210, rx_lane_map=0x0123),
        _fab(label="2",  serdes=( 64,  65,  66,  67),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0000,
              tx_lane_map=0x0123, rx_lane_map=0x3210),
        _fab(label="3",  serdes=( 76,  77,  78,  79),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x000f,
              tx_lane_map=0x1302, rx_lane_map=0x1302),
        _fab(label="4",  serdes=( 60,  61,  62,  63),
              tx_polarity_flip=0x000f, rx_polarity_flip=0x0000,
              tx_lane_map=0x2310, rx_lane_map=0x1302),
        _fab(label="5",  serdes=( 56,  57,  58,  59),
              tx_polarity_flip=0x000f, rx_polarity_flip=0x0000,
              tx_lane_map=0x0123, rx_lane_map=0x3210),
        _fab(label="6",  serdes=( 52,  53,  54,  55),
              tx_polarity_flip=0x000f, rx_polarity_flip=0x0000,
              tx_lane_map=0x0123, rx_lane_map=0x3210),
        _fab(label="7",  serdes=( 48,  49,  50,  51),
              tx_polarity_flip=0x000f, rx_polarity_flip=0x0000,
              tx_lane_map=0x0123, rx_lane_map=0x3210),

        # Fabric Card Tomahawk to lc #2
        _fab(label="8",  serdes=(104, 105, 106, 107),
              tx_polarity_flip=0x000f, rx_polarity_flip=0x0000,
              tx_lane_map=0x3210, rx_lane_map=0x2301),
        _fab(label="9",  serdes=(100, 101, 102, 103),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x000f,
              tx_lane_map=0x0123, rx_lane_map=0x1032),
        _fab(label="10", serdes=( 96,  97,  98,  99),
              tx_polarity_flip=0x000f, rx_polarity_flip=0x0000,
              tx_lane_map=0x3210, rx_lane_map=0x2301),
        _fab(label="11", serdes=(108, 109, 110, 111),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x000f,
              tx_lane_map=0x0123, rx_lane_map=0x0213),
        _fab(label="12", serdes=( 92,  93,  94,  95),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0000,
              tx_lane_map=0x1023, rx_lane_map=0x0213),
        _fab(label="13", serdes=( 88,  89,  90,  91),
              tx_polarity_flip=0x000f, rx_polarity_flip=0x000f,
              tx_lane_map=0x0123, rx_lane_map=0x1032),
        _fab(label="14", serdes=( 80,  81,  82,  83),
              tx_polarity_flip=0x000f, rx_polarity_flip=0x0000,
              tx_lane_map=0x0123, rx_lane_map=0x1032),
        _fab(label="15", serdes=( 84,  85,  86,  87),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x000f,
              tx_lane_map=0x3210, rx_lane_map=0x2301),

        # Fabric Card Tomahawk to lc #3
        _fab(label="16", serdes=(  4,   5,   6,   7),
              tx_polarity_flip=0x000f, rx_polarity_flip=0x0000,
              tx_lane_map=0x0123, rx_lane_map=0x3210),
        _fab(label="17", serdes=(  8,   9,  10,  11),
              tx_polarity_flip=0x000f, rx_polarity_flip=0x0000,
              tx_lane_map=0x0123, rx_lane_map=0x3210),
        _fab(label="18", serdes=(  0,   1,   2,   3),
              tx_polarity_flip=0x000f, rx_polarity_flip=0x000f,
              tx_lane_map=0x0123, rx_lane_map=0x3210),
        _fab(label="19", serdes=( 12,  13,  14,  15),
              tx_polarity_flip=0x000f, rx_polarity_flip=0x0000,
              tx_lane_map=0x1302, rx_lane_map=0x3120),
        _fab(label="20", serdes=(124, 125, 126, 127),
              tx_polarity_flip=0x000f, rx_polarity_flip=0x0000,
              tx_lane_map=0x2310, rx_lane_map=0x2301),
        _fab(label="21", serdes=(112, 113, 114, 115),
              tx_polarity_flip=0x000f, rx_polarity_flip=0x0000,
              tx_lane_map=0x0123, rx_lane_map=0x1032),
        _fab(label="22", serdes=(116, 117, 118, 119),
              tx_polarity_flip=0x000f, rx_polarity_flip=0x0000,
              tx_lane_map=0x3210, rx_lane_map=0x0123),
        _fab(label="23", serdes=(120, 121, 122, 123),
              tx_polarity_flip=0x000f, rx_polarity_flip=0x0000,
              tx_lane_map=0x3210, rx_lane_map=0x0123),

        # Fabric Card Tomahawk to lc #4
        _fab(label="24", serdes=( 40,  41,  42,  43),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x000f,
              tx_lane_map=0x0123, rx_lane_map=0x0123),
        _fab(label="25", serdes=( 36,  37,  38,  39),
              tx_polarity_flip=0x000f, rx_polarity_flip=0x0000,
              tx_lane_map=0x3210, rx_lane_map=0x3210),
        _fab(label="26", serdes=( 28,  29,  30,  31),
              tx_polarity_flip=0x000f, rx_polarity_flip=0x0000,
              tx_lane_map=0x3210, rx_lane_map=0x3210),
        _fab(label="27", serdes=( 44,  45,  46,  47),
              tx_polarity_flip=0x000f, rx_polarity_flip=0x0000,
              tx_lane_map=0x2031, rx_lane_map=0x2031),
        _fab(label="28", serdes=( 32,  33,  34,  35),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x000f,
              tx_lane_map=0x1023, rx_lane_map=0x0213),
        _fab(label="29", serdes=( 20,  21,  22,  23),
              tx_polarity_flip=0x000f, rx_polarity_flip=0x0000,
              tx_lane_map=0x0123, rx_lane_map=0x0123),
        _fab(label="30", serdes=( 24,  25,  26,  27),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x000f,
              tx_lane_map=0x3210, rx_lane_map=0x3210),
        _fab(label="31", serdes=( 16,  17,  18,  19),
              tx_polarity_flip=0x0002, rx_polarity_flip=0x000f,
              tx_lane_map=0x3210, rx_lane_map=0x3210)
    )


class CelBigStoneG100Fab1(CelBigStoneG100):
    name = 'cel,bigstone_g_fab1'
    chip = cumulus.platform.TomahawkChip()
    switch = CelBigStoneG100Switch(chip)

Platform = CelBigStoneG100Fab1
