#
# Copyright (C) 2017 Cumulus Networks, Inc. All rights reserved
#
# This software is subject to the Cumulus Networks End User License
# Agreement available  at the following locations:
#
# Internet: https://cumulusnetworks.com/downloads/eula/latest/view/
# Cumulus Linux systems: /usr/share/cumulus/EULA.txt
#

import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors
from cumulus.platforms.cel_smallstone_xp import CelSmallstoneXpLED, CelSmallstoneXpQSFPp

vhwmon_path = '/sys/class/misc/vhwmon/bmc:'

class CelSmallstoneXpbCoreTemp(cumulus.sensors.Temp_Unit):
    description = cumulus.sensors.Temp_Unit.intel_cpu_sensor_name
    driver_path = '/sys/devices/platform/coretemp.0'
    temp_max = 82
    temp_crit = 104
    no_shutdown = True

class CelSmallstoneXpbTemp1(CelSmallstoneXpbCoreTemp):
    name = 'Temp1'
    description = 'Physical id 0'
    driver_hwmon = ['temp1']

class CelSmallstoneXpbTemp2(CelSmallstoneXpbCoreTemp):
    name = 'Temp2'
    description = 'Core 0'
    driver_hwmon = ['temp2']

class CelSmallstoneXpbTemp3(CelSmallstoneXpbCoreTemp):
    name = 'Temp3'
    description = 'Core 1'
    driver_hwmon = ['temp3']

class CelSmallstoneXpbSysTemp(cumulus.sensors.Temp_Unit):
    driver_path = vhwmon_path + 'temp'

class CelSmallstoneXpbTemp4(CelSmallstoneXpbSysTemp):
    name = 'Temp4'
    description = 'Base_Temp_U5'
    temp_crit = 100
    temp_max = 95
    driver_hwmon = ['temp1']

class CelSmallstoneXpbTemp5(CelSmallstoneXpbSysTemp):
    name = 'Temp5'
    description = 'Base_Temp_U7'
    temp_crit = 55
    temp_max = 50
    driver_hwmon = ['temp2']

class CelSmallstoneXpbTemp6(CelSmallstoneXpbSysTemp):
    name = 'Temp6'
    description = 'SXP_Temp_U8'
    temp_crit = 80
    temp_max = 74
    driver_hwmon = ['temp3']

class CelSmallstoneXpbTemp7(CelSmallstoneXpbSysTemp):
    name = 'Temp7'
    description = 'SXP_Temp_U28'
    temp_crit = 55
    temp_max = 50
    driver_hwmon = ['temp4']

class CelSmallstoneXpbTemp8(CelSmallstoneXpbSysTemp):
    name = 'Temp8'
    description = 'SXP_Temp_U1'
    temp_crit = 80
    temp_max = 74
    driver_hwmon = ['temp5']

class CelSmallstoneXpbTemp9(CelSmallstoneXpbSysTemp):
    name = 'Temp9'
    description = 'SXP_Temp_U2'
    temp_crit = 50
    temp_max = 55
    driver_hwmon = ['temp6']

class CelSmallstoneXpbTemp10(CelSmallstoneXpbSysTemp):
    name = 'Temp10'
    description = 'Switch_U26_temp'
    temp_crit = 55
    temp_max = 50
    driver_hwmon = ['temp7']

class CelSmallstoneXpbTemp11(CelSmallstoneXpbSysTemp):
    name = 'Temp11'
    description = 'Switch_U13_temp'
    temp_crit = 55
    temp_max = 50
    driver_hwmon = ['temp8']

class CelSmallstoneXpb_PSU(cumulus.sensors.PSU_Unit):
    driver_hwmon = ['power1']

class CelSmallstoneXpb_PSU1(CelSmallstoneXpb_PSU):
    name = 'PSU1'
    driver_path = vhwmon_path + 'psul'
    psu_ok = 0

    def get_status(self):
        val = self.attrs.get('power1_input', 0)
        if val != 0:
            val = 1
        self.attrs['psu_pwr1_all_ok'] = val
        CelSmallstoneXpb_PSU1.psu_ok = val
        return cumulus.sensors.PSU_Unit.get_status(self)

class CelSmallstoneXpb_PSU2(CelSmallstoneXpb_PSU):
    name = 'PSU2'
    driver_path = vhwmon_path + 'psur'
    psu_ok = 0

    def get_status(self):
        val = self.attrs.get('power1_input', 0)
        if val != 0:
            val = 1
        self.attrs['psu_pwr1_all_ok'] = val
        CelSmallstoneXpb_PSU2.psu_ok = val
        return cumulus.sensors.PSU_Unit.get_status(self)

class CelSmallstoneXpbPsuTemp(cumulus.sensors.Temp_Unit):
    driver_hwmon = ['temp1']

    def detect_state_plat(self):
        if not self.psu.psu_ok:
            return 'ABSENT'
        return cumulus.sensors.Temp_Unit.detect_state_plat(self)

class CelSmallstoneXpbTemp12(CelSmallstoneXpbPsuTemp):
    name = 'PSU1Temp1'
    description = 'PSU1 Temp1'
    psu = CelSmallstoneXpb_PSU1()
    driver_path = psu.driver_path

class CelSmallstoneXpbTemp13(CelSmallstoneXpbPsuTemp):
    name = 'PSU2Temp1'
    description = 'PSU2 Temp1'
    psu = CelSmallstoneXpb_PSU2()
    driver_path = psu.driver_path

class CelSmallstoneXpbSysFan(cumulus.sensors.Fan_Unit):
    driver_path = vhwmon_path + 'fan'
    no_pwm = True

class CelSmallstoneXpbFan1(CelSmallstoneXpbSysFan):
    name = 'Fan1'
    description = 'Fan 1 Rear'
    driver_hwmon = ['fan1']

class CelSmallstoneXpbFan2(CelSmallstoneXpbSysFan):
    name = 'Fan2'
    description = 'Fan 1 Front'
    driver_hwmon = ['fan2']

class CelSmallstoneXpbFan3(CelSmallstoneXpbSysFan):
    name = 'Fan3'
    description = 'Fan 2 Rear'
    driver_hwmon = ['fan3']

class CelSmallstoneXpbFan4(CelSmallstoneXpbSysFan):
    name = 'Fan4'
    description = 'Fan 2 Front'
    driver_hwmon = ['fan4']

class CelSmallstoneXpbFan5(CelSmallstoneXpbSysFan):
    name = 'Fan5'
    description = 'Fan 3 Rear'
    driver_hwmon = ['fan5']

class CelSmallstoneXpbFan6(CelSmallstoneXpbSysFan):
    name = 'Fan6'
    description = 'Fan 3 Front'
    driver_hwmon = ['fan6']

class CelSmallstoneXpbFan7(CelSmallstoneXpbSysFan):
    name = 'Fan7'
    description = 'Fan 4 Rear'
    driver_hwmon = ['fan7']

class CelSmallstoneXpbFan8(CelSmallstoneXpbSysFan):
    name = 'Fan8'
    description = 'Fan 4 Front'
    driver_hwmon = ['fan8']

class CelSmallstoneXpbPsuFan(cumulus.sensors.Fan_Unit):
    driver_hwmon = ['fan1']
    no_pwm = True

    def get_status(self):
        if not self.psu.psu_ok:
            return self.update_status(None, 'not_ok')
        return cumulus.sensors.Fan_Unit.get_status(self)

    def detect_state_plat(self):
        if not self.psu.psu_ok:
            return 'ABSENT'
        return cumulus.sensors.Fan_Unit.detect_state_plat(self)

class CelSmallstoneXpbFan9(CelSmallstoneXpbPsuFan):
    name = 'PSU1Fan1'
    description = 'PSU1 Fan1'
    psu = CelSmallstoneXpb_PSU1()
    driver_path = psu.driver_path

class CelSmallstoneXpbFan10(CelSmallstoneXpbPsuFan):
    name = 'PSU2Fan1'
    description = 'PSU2 Fan1'
    psu = CelSmallstoneXpb_PSU2()
    driver_path = psu.driver_path

class CelSmallstoneXpbSensors():
    units = (
        CelSmallstoneXpbTemp1(),
        CelSmallstoneXpbTemp2(),
        CelSmallstoneXpbTemp3(),
        CelSmallstoneXpbTemp4(),
        CelSmallstoneXpbTemp5(),
        CelSmallstoneXpbTemp6(),
        CelSmallstoneXpbTemp7(),
        CelSmallstoneXpbTemp8(),
        CelSmallstoneXpbTemp9(),
        CelSmallstoneXpbTemp10(),
        CelSmallstoneXpbTemp11(),
        CelSmallstoneXpbTemp12(),
        CelSmallstoneXpbTemp13(),
        CelSmallstoneXpbFan1(),
        CelSmallstoneXpbFan2(),
        CelSmallstoneXpbFan3(),
        CelSmallstoneXpbFan4(),
        CelSmallstoneXpbFan5(),
        CelSmallstoneXpbFan6(),
        CelSmallstoneXpbFan7(),
        CelSmallstoneXpbFan8(),
        CelSmallstoneXpbFan9(),
        CelSmallstoneXpbFan10(),
        CelSmallstoneXpb_PSU1(),
        CelSmallstoneXpb_PSU2(),
    )

class CelSmallstoneXpbBMC(cumulus.platform.BMC):
    name = 'bmc'
    type = 'ipmibmc'
    ignores = ['RXP_Temp_U8', 'RXP_Temp_U2', 'RXP_Temp_U23', 'RXP_Temp_U22',
               'Fan_3_Rear', 'Fan_3_Front', 'PSU2_Power_In', 'TempCOM-E']
    renames = { 'Fan_4_Rear': 'Fan_3_Rear', 'Fan_4_Front': 'Fan_3_Front',
                'Fan_5_Rear': 'Fan_4_Rear', 'Fan_5_Front': 'Fan_4_Front'}


class CelSmallstoneXpbSwitch(cumulus.platform.BCMSwitch):
    ledup = CelSmallstoneXpLED
    _qsfp = CelSmallstoneXpQSFPp

    has_phy_ucode_mdio = False
    has_phy_ucode_spi_rom = False

    ports = (
        _qsfp(label="1", serdes=( 16,  17,  18,  19), leds=(0, 0),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0008,
              rx_lane_map=0x0132, tx_lane_map=0x1032),
        _qsfp(label="2", serdes=( 20,  21,  22,  23), leds=(0, 8),
              tx_polarity_flip=0x000a, rx_polarity_flip=0x0007,
              rx_lane_map=0x2031, tx_lane_map=0x1230),
        _qsfp(label="3", serdes=( 24,  25,  26,  27), leds=(0, 16),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0008,
              rx_lane_map=0x0132, tx_lane_map=0x1032),
        _qsfp(label="4", serdes=( 28,  29,  30,  31), leds=(0, 24),
              tx_polarity_flip=0x000a, rx_polarity_flip=0x0007,
              rx_lane_map=0x2031, tx_lane_map=0x1230),
        _qsfp(label="5", serdes=( 32,  33,  34,  35), leds=(0, 32),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0008,
              rx_lane_map=0x0132, tx_lane_map=0x1032),
        _qsfp(label="6", serdes=( 36,  37,  38,  39), leds=(0, 40),
              tx_polarity_flip=0x000a, rx_polarity_flip=0x0007,
              rx_lane_map=0x2031, tx_lane_map=0x1230),
        _qsfp(label="7", serdes=( 40,  41,  42,  43), leds=(0, 48),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0008,
              rx_lane_map=0x0132, tx_lane_map=0x1032),
        _qsfp(label="8", serdes=( 44,  45,  46,  47), leds=(0, 56),
              tx_polarity_flip=0x000A, rx_polarity_flip=0x0007,
              tx_lane_map=0x1230, rx_lane_map=0x2031),
        _qsfp(label="9", serdes=(  0,   1,   2,   3), leds=(0, 64),
              tx_polarity_flip=0x000F, rx_polarity_flip=0x0005,
              tx_lane_map=0x0132, rx_lane_map=0x3102),
        _qsfp(label="10", serdes=(  4,   5,   6,   7), leds=(0, 72),
              tx_polarity_flip=0x0005, rx_polarity_flip=0x000F,
              tx_lane_map=0x1302, rx_lane_map=0x0321),
        _qsfp(label="11", serdes=(  8,   9,  10,  11), leds=(0, 80),
              tx_polarity_flip=0x000A, rx_polarity_flip=0x0007,
              tx_lane_map=0x2031, rx_lane_map=0x2310),
        _qsfp(label="12", serdes=( 12,  13,  14,  15), leds=(0, 88),
              tx_polarity_flip=0x0005, rx_polarity_flip=0x000F,
              tx_lane_map=0x2310, rx_lane_map=0x2310),
        _qsfp(label="13", serdes=( 48,  49,  50,  51), leds=(0, 96),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x000B,
              tx_lane_map=0x1230, rx_lane_map=0x0132),
        _qsfp(label="14", serdes=( 52,  53,  54,  55), leds=(0, 104),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0002,
              tx_lane_map=0x1230, rx_lane_map=0x2031),
        _qsfp(label="15", serdes=( 56,  57,  58,  59), leds=(0, 112),
              tx_polarity_flip=0x000e, rx_polarity_flip=0x0008,
              tx_lane_map=0x1032, rx_lane_map=0x0132),
        _qsfp(label="16", serdes=( 60,  61,  62,  63), leds=(0, 120),
              tx_polarity_flip=0x000a, rx_polarity_flip=0x0002,
              tx_lane_map=0x1203, rx_lane_map=0x2031),
        _qsfp(label="17", serdes=( 64,  65,  66,  67), leds=(1, 0),
              tx_polarity_flip=0x000f, rx_polarity_flip=0x000c,
              tx_lane_map=0x1032, rx_lane_map=0x0132),
        _qsfp(label="18", serdes=( 68,  69,  70,  71), leds=(1, 8),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0002,
              tx_lane_map=0x1302, rx_lane_map=0x2031),
        _qsfp(label="19", serdes=( 72,  73,  74,  75), leds=(1, 16),
              tx_polarity_flip=0x0005, rx_polarity_flip=0x0007,
              tx_lane_map=0x1230, rx_lane_map=0x0132),
        _qsfp(label="20", serdes=( 76,  77,  78,  79), leds=(1, 24),
              tx_polarity_flip=0x0005, rx_polarity_flip=0x0002,
              tx_lane_map=0x0321, rx_lane_map=0x2031),
        _qsfp(label="21", serdes=(112, 113, 114, 115), leds=(1, 32),
              tx_polarity_flip=0x000f, rx_polarity_flip=0x0005,
              tx_lane_map=0x0132, rx_lane_map=0x3102),
        _qsfp(label="22", serdes=(116, 117, 118, 119), leds=(1, 40),
              tx_polarity_flip=0x000a, rx_polarity_flip=0x0005,
              tx_lane_map=0x1302, rx_lane_map=0x0321),
        _qsfp(label="23", serdes=(120, 121, 122, 123), leds=(1, 48),
              tx_polarity_flip=0x0005, rx_polarity_flip=0x000f,
              tx_lane_map=0x2031, rx_lane_map=0x2103),
        _qsfp(label="24", serdes=(124, 125, 126, 127), leds=(1, 56),
              tx_polarity_flip=0x000f, rx_polarity_flip=0x0005,
              tx_lane_map=0x3201, rx_lane_map=0x2013),
        _qsfp(label="25", serdes=( 80,  81,  82,  83), leds=(1, 64),
              tx_polarity_flip=0x000f, rx_polarity_flip=0x0007,
              tx_lane_map=0x0321, rx_lane_map=0x0132),
        _qsfp(label="26", serdes=( 84,  85,  86,  87), leds=(1, 72),
              tx_polarity_flip=0x0005, rx_polarity_flip=0x0000,
              tx_lane_map=0x0321, rx_lane_map=0x2310),
        _qsfp(label="27", serdes=( 88,  89,  90,  91), leds=(1, 80),
              tx_polarity_flip=0x000f, rx_polarity_flip=0x0007,
              tx_lane_map=0x0321, rx_lane_map=0x0132),
        _qsfp(label="28", serdes=( 92,  93,  94,  95), leds=(1, 88),
              tx_polarity_flip=0x0005, rx_polarity_flip=0x0000,
              tx_lane_map=0x0321, rx_lane_map=0x2310),
        _qsfp(label="29", serdes=( 96,  97,  98,  99), leds=(1, 96),
              tx_polarity_flip=0x000f, rx_polarity_flip=0x0007,
              tx_lane_map=0x0321, rx_lane_map=0x0132),
        _qsfp(label="30", serdes=(100, 101, 102, 103), leds=(1, 104),
              tx_polarity_flip=0x0005, rx_polarity_flip=0x0000,
              tx_lane_map=0x0321, rx_lane_map=0x2310),
        _qsfp(label="31", serdes=(104, 105, 106, 107), leds=(1, 112),
              tx_polarity_flip=0x000f, rx_polarity_flip=0x0007,
              tx_lane_map=0x0321, rx_lane_map=0x0132),
        _qsfp(label="32", serdes=(108, 109, 110, 111), leds=(1, 120),
              tx_polarity_flip=0x0005, rx_polarity_flip=0x0000,
              tx_lane_map=0x0321, rx_lane_map=0x2310),
    )


class CelSmallstoneXp(cumulus.platform.Platform):
    name = 'cel,smallstone_xp_b'
    chip = cumulus.platform.TridentTwo_56850_Chip()
    switch = CelSmallstoneXpbSwitch(chip)
    sensors = CelSmallstoneXpbSensors()
    bmc = CelSmallstoneXpbBMC()
    cpld = None
    statusleds = None
    pwms = None

Platform = CelSmallstoneXp
