#
# Copyright 2016.  Cumulus Networks, Inc.
#

import cumulus.platform
import cumulus.cpld
import cumulus.sensors

CumulusVXPath = '/sys/class/hwmon/hwmon0'

class CumulusVXCPLD(cumulus.cpld.SysFS):
    path = CumulusVXPath

class CumulusVXTemp1(cumulus.sensors.Temp_Unit):
    name = 'Temp1'
    description = 'Board Sensor near CPU'
    driver_path = CumulusVXPath
    driver_hwmon = ['temp1']

class CumulusVXTemp2(cumulus.sensors.Temp_Unit):
    name = 'Temp2'
    description = 'Board Sensor Near Virtual Switch'
    driver_path = CumulusVXPath
    driver_hwmon = ['temp2']

class CumulusVXTemp3(cumulus.sensors.Temp_Unit):
    name = 'Temp3'
    description = 'Board Sensor at Front Left Corner'
    driver_path = CumulusVXPath
    driver_hwmon = ['temp3']

class CumulusVXTemp4(cumulus.sensors.Temp_Unit):
    name = 'Temp4'
    description = 'Board Sensor at Front Right Corner'
    driver_path = CumulusVXPath
    driver_hwmon = ['temp4']

class CumulusVXTemp5(cumulus.sensors.Temp_Unit):
    name = 'Temp5'
    description = 'Board Sensor near Fan'
    driver_path = CumulusVXPath
    driver_hwmon = ['temp5']

class CumulusVXTemp6(cumulus.sensors.Temp_Unit):
    name = 'PSU1Temp1'
    description = 'PSU1 Temp Sensor'
    driver_path = CumulusVXPath
    driver_hwmon = ['temp6']

class CumulusVXTemp7(cumulus.sensors.Temp_Unit):
    name = 'PSU2Temp1'
    description = 'PSU2 Temp Sensor'
    driver_path = CumulusVXPath
    driver_hwmon = ['temp7']

class CumulusVXFan1(cumulus.sensors.Fan_Unit):
    name = 'Fan1'
    description = 'Fan Tray 1, Fan 1'
    driver_path = CumulusVXPath
    driver_hwmon = ['fan1']
    cpld_hwmon = ['fan1']
    rpmmode = 1

class CumulusVXFan2(cumulus.sensors.Fan_Unit):
    name = 'Fan2'
    description = 'Fan Tray 1, Fan 2'
    driver_path = CumulusVXPath
    driver_hwmon = ['fan2']
    cpld_hwmon = ['fan2']
    rpmmode = 1

class CumulusVXFan3(cumulus.sensors.Fan_Unit):
    name = 'Fan3'
    description = 'Fan Tray 2, Fan 1'
    driver_path = CumulusVXPath
    driver_hwmon = ['fan3']
    cpld_hwmon = ['fan3']
    rpmmode = 1

class CumulusVXFan4(cumulus.sensors.Fan_Unit):
    name = 'Fan4'
    description = 'Fan Tray 2, Fan 2'
    driver_path = CumulusVXPath
    driver_hwmon = ['fan4']
    cpld_hwmon = ['fan4']
    rpmmode = 1

class CumulusVXFan5(cumulus.sensors.Fan_Unit):
    name = 'Fan5'
    description = 'Fan Tray 3, Fan 1'
    driver_path = CumulusVXPath
    driver_hwmon = ['fan5']
    cpld_hwmon = ['fan5']
    rpmmode = 1

class CumulusVXFan6(cumulus.sensors.Fan_Unit):
    name = 'Fan6'
    description = 'Fan Tray 3, Fan 2'
    driver_path = CumulusVXPath
    driver_hwmon = ['fan6']
    cpld_hwmon = ['fan6']
    rpmmode = 1

class CumulusVXFan7(cumulus.sensors.Fan_Unit):
    name = 'PSU1Fan1'
    description = 'PSU1 Fan'
    driver_path = CumulusVXPath
    driver_hwmon = ['fan7']
    cpld_hwmon = ['fan7']
    rpmmode = 1

class CumulusVXFan8(cumulus.sensors.Fan_Unit):
    name = 'PSU2Fan1'
    description = 'PSU2 Fan'
    driver_path = CumulusVXPath
    driver_hwmon = ['fan8']
    cpld_hwmon = ['fan8']
    rpmmode = 1

class CumulusVXPSU1(cumulus.sensors.PSU_Unit):
    name = 'PSU1'
    cpld_hwmon = ['psu1']

class CumulusVXPSU2(cumulus.sensors.PSU_Unit):
    name = 'PSU2'
    cpld_hwmon = ['psu2']

class CumulusVXPowerLED(cumulus.sensors.StatusLED_Unit):
    name = 'Power'
    led_location = 'led_psu'
    sources = ['PSU1', 'PSU2']
    bad_led_color = 'yellow'

class CumulusVXFanLED(cumulus.sensors.StatusLED_Unit):
    name = 'Fan'
    led_location = 'led_fan'
    sources = ['Fan1', 'Fan2', 'Fan3', 'Fan4', 'Fan5', 'Fan6' ]
    bad_led_color = 'yellow'

class CumulusVXSystemLED(cumulus.sensors.StatusLED_Unit):
    name = 'System'
    led_location = 'led_system'

class CumulusVXPWM1(cumulus.sensors.PWM_Unit):
    name = 'System PWM'
    fans = (CumulusVXFan1(),
            CumulusVXFan2(),
            CumulusVXFan3(),
            CumulusVXFan4(),
            CumulusVXFan5(),
            CumulusVXFan6(),
            CumulusVXFan7(),
            CumulusVXFan8(),
            )
    temps = (CumulusVXTemp1(),
             CumulusVXTemp2(),
             CumulusVXTemp3(),
             CumulusVXTemp4(),
             CumulusVXTemp5(),
             CumulusVXTemp6(),
             CumulusVXTemp7(),
            )

class CumulusVXPWMs():
    units = (CumulusVXPWM1(),)

class CumulusVXSensors():
    units = (CumulusVXFan1(),
             CumulusVXFan2(),
             CumulusVXFan3(),
             CumulusVXFan4(),
             CumulusVXFan5(),
             CumulusVXFan6(),
             CumulusVXFan7(),
             CumulusVXFan8(),
             CumulusVXTemp1(),
             CumulusVXTemp2(),
             CumulusVXTemp3(),
             CumulusVXTemp4(),
             CumulusVXTemp5(),
             CumulusVXTemp6(),
             CumulusVXTemp7(),
             CumulusVXPSU1(),
             CumulusVXPSU2(),
            )

class CumulusVXLEDs():
    units = (CumulusVXPowerLED(),
             CumulusVXFanLED(),
             CumulusVXSystemLED(),
             )

class CumulusVX(cumulus.platform.Platform):
    name = 'cumulus,vx'
    chip = cumulus.platform.VXChip()
    switch = cumulus.platform.VXSwitch(chip)
    cpld = CumulusVXCPLD()
    statusleds = CumulusVXLEDs()
    pwms = CumulusVXPWMs()
    sensors = CumulusVXSensors()

Platform = CumulusVX
