#
# Copyright 2016.  Cumulus Networks, Inc.
#

import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors
import cumulus.platforms.mlnx_SN2xxx


class MlxSN2410BSFP(cumulus.platform.Spectrum10GPort):
    _set_led = '''
;;; MlxSN2410BSFP
'''
    def __init__(self, label, adj_label, phy_addr, hw_port):
        hw_port_name = 'xe%d' % hw_port

        cumulus.platform.Spectrum10GPort.__init__(self, label=label,
                        adj_label=adj_label, hw_port=hw_port_name,
                        phy_addr=phy_addr)
        self.sfp = cumulus.sfp.SFPROMSDK(hw_port, 'sfp28')

    def set_leds(self, mode, led_port, sub):
        self._check_mode(mode)

        set_calls = {}

        return set_calls

class MlxSN2410BQSFP(cumulus.platform.Spectrum100GPort):
    _set_led = '''
;;; MlxSN2410BQSFP
;;; Two bits per port: yellow(active high), green(active high)
'''
    def __init__(self, label, adj_label, phy_addrs, hw_ports,
                 blocked_adj_split=None):

        hw_port_names = []
        for hw_port in hw_ports:
            hw_port_names.append('xe%d' % hw_port)

        cumulus.platform.Spectrum100GPort.__init__(self, label=label,
                        adj_label=adj_label, hw_ports=tuple(hw_port_names),
                        phy_addrs=phy_addrs, blocked_adj_split=blocked_adj_split)
        self.sfp = cumulus.sfp.SFPROMSDK(hw_ports[0], 'qsfp28')

    def set_leds(self, mode, led_port, sub):
        self._check_mode(mode)

        set_calls = {}

        return set_calls

class MlxSN2410BSwitch(cumulus.platform.SpectrumSwitch):
    _sfp28 = MlxSN2410BSFP
    _qsfp28 = MlxSN2410BQSFP

    has_phy_ucode_mdio = False
    has_phy_ucode_spi_rom = False

    # label is front panel port label.
    # adj_label is front panel port label of the adjacent port (because the
    #    adjacent label is not necessarily label+1). Note that adj_label
    #    is meaningful only when blocked_adj_split is specified.
    #
    # phy_addrs correspond to XML lane-to-module.
    # hw_ports correspond to XML local-port; the length of the tuple
    #    (when > 1) implies the max number of ports into which this port can be split.
    # blocked_adj_split Port is blocked when its adjacent port is split into
    #    this number of ports (or more)
    #
    ports = (
        _sfp28(label="1",  adj_label=None, phy_addr=0,      hw_port=61),
        _sfp28(label="2",  adj_label=None, phy_addr=1,      hw_port=62),
        _sfp28(label="3",  adj_label=None, phy_addr=2,      hw_port=63),
        _sfp28(label="4",  adj_label=None, phy_addr=3,      hw_port=64),
        _sfp28(label="5",  adj_label=None, phy_addr=4,      hw_port=57),
        _sfp28(label="6",  adj_label=None, phy_addr=5,      hw_port=58),
        _sfp28(label="7",  adj_label=None, phy_addr=6,      hw_port=59),
        _sfp28(label="8",  adj_label=None, phy_addr=7,      hw_port=60),
        _sfp28(label="9",  adj_label=None, phy_addr=8,      hw_port=53),
        _sfp28(label="10", adj_label=None, phy_addr=9,      hw_port=54),
        _sfp28(label="11", adj_label=None, phy_addr=10,     hw_port=55),
        _sfp28(label="12", adj_label=None, phy_addr=11,     hw_port=56),
        _sfp28(label="13", adj_label=None, phy_addr=12,     hw_port=49),
        _sfp28(label="14", adj_label=None, phy_addr=13,     hw_port=50),
        _sfp28(label="15", adj_label=None, phy_addr=14,     hw_port=51),
        _sfp28(label="16", adj_label=None, phy_addr=15,     hw_port=52),
        _sfp28(label="17", adj_label=None, phy_addr=16,     hw_port=45),
        _sfp28(label="18", adj_label=None, phy_addr=17,     hw_port=46),
        _sfp28(label="19", adj_label=None, phy_addr=18,     hw_port=47),
        _sfp28(label="20", adj_label=None, phy_addr=19,     hw_port=48),
        _sfp28(label="21", adj_label=None, phy_addr=20,     hw_port=41),
        _sfp28(label="22", adj_label=None, phy_addr=21,     hw_port=42),
        _sfp28(label="23", adj_label=None, phy_addr=22,     hw_port=43),
        _sfp28(label="24", adj_label=None, phy_addr=23,     hw_port=44),
        _sfp28(label="25", adj_label=None, phy_addr=24,     hw_port=37),
        _sfp28(label="26", adj_label=None, phy_addr=25,     hw_port=38),
        _sfp28(label="27", adj_label=None, phy_addr=26,     hw_port=39),
        _sfp28(label="28", adj_label=None, phy_addr=27,     hw_port=40),
        _sfp28(label="29", adj_label=None, phy_addr=28,     hw_port=33),
        _sfp28(label="30", adj_label=None, phy_addr=29,     hw_port=34),
        _sfp28(label="31", adj_label=None, phy_addr=30,     hw_port=35),
        _sfp28(label="32", adj_label=None, phy_addr=31,     hw_port=36),
        _sfp28(label="33", adj_label=None, phy_addr=32,     hw_port=1),
        _sfp28(label="34", adj_label=None, phy_addr=33,     hw_port=2),
        _sfp28(label="35", adj_label=None, phy_addr=34,     hw_port=3),
        _sfp28(label="36", adj_label=None, phy_addr=35,     hw_port=4),
        _sfp28(label="37", adj_label=None, phy_addr=36,     hw_port=5),
        _sfp28(label="38", adj_label=None, phy_addr=37,     hw_port=6),
        _sfp28(label="39", adj_label=None, phy_addr=38,     hw_port=7),
        _sfp28(label="40", adj_label=None, phy_addr=39,     hw_port=8),
        _sfp28(label="41", adj_label=None, phy_addr=40,     hw_port=9),
        _sfp28(label="42", adj_label=None, phy_addr=41,     hw_port=10),
        _sfp28(label="43", adj_label=None, phy_addr=42,     hw_port=11),
        _sfp28(label="44", adj_label=None, phy_addr=43,     hw_port=12),
        _sfp28(label="45", adj_label=None, phy_addr=44,     hw_port=13),
        _sfp28(label="46", adj_label=None, phy_addr=45,     hw_port=14),
        _sfp28(label="47", adj_label=None, phy_addr=46,     hw_port=15),
        _sfp28(label="48", adj_label=None, phy_addr=47,     hw_port=16),

        _qsfp28(label="49", adj_label=None, phy_addrs=(48,48,48,48), hw_ports=(17,18,19,20)),
        _qsfp28(label="50", adj_label="49", phy_addrs=(49,49,49,49), hw_ports=(19,20), blocked_adj_split=4),
        _qsfp28(label="51", adj_label=None, phy_addrs=(50,50,50,50), hw_ports=(21,22,23,24)),
        _qsfp28(label="52", adj_label="51", phy_addrs=(51,51,51,51), hw_ports=(23,24), blocked_adj_split=4),
        _qsfp28(label="53", adj_label=None, phy_addrs=(52,52,52,52), hw_ports=(25,26,27,28)),
        _qsfp28(label="54", adj_label="53", phy_addrs=(53,53,53,53), hw_ports=(27,28), blocked_adj_split=4),
        _qsfp28(label="55", adj_label=None, phy_addrs=(54,54,54,54), hw_ports=(29,30,31,32)),
        _qsfp28(label="56", adj_label="55", phy_addrs=(55,55,55,55), hw_ports=(31,32), blocked_adj_split=4),
    )

class MlxSN2410B(cumulus.platform.Platform):
    name = 'mlnx,x86_MSN2410B'
    chip = cumulus.platform.SpectrumChip()
    switch = MlxSN2410BSwitch(chip)
    cpld = cumulus.platforms.mlnx_SN2xxx.Mlx2xxxCPLD()
    sensors = cumulus.platforms.mlnx_SN2xxx.MlxSN2xxxSensors('mlnx-asic-drv-54')
    pwms = cumulus.platforms.mlnx_SN2xxx.MlxSN2xxxPWMs('mlnx-asic-drv-54')
    statusleds = cumulus.platforms.mlnx_SN2xxx.MlxSN2xxxLEDs()

Platform = MlxSN2410B
