#
# Copyright 2016.  Cumulus Networks, Inc.
#
import os
import subprocess
import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors

class QuantaIX1_Fan(cumulus.sensors.Fan_Unit):
    driver_path = ''
    pci_path = '/sys/devices/pci0000:00/0000:00:1f.3'
    if os.path.exists(pci_path):
        for x in os.listdir(pci_path):
            if x.startswith('i2c-'):
                a,b = x.split('-')
                c = b + '-004e'
                driver_path = os.path.join(pci_path, x, c)
                break

    fan_min = 9300
    fan_max = 23000
    rpmmode = 1
    kick = True

    def find_and_add_attrs(self):
        if self.get_gpio_value(self.fan_present) == '0':
            self.attrs['fan_0'] = 'installed, ok'
        else:
            self.attrs['fan_0'] = 'not_installed'
        return cumulus.sensors.Fan_Unit.find_and_add_attrs(self)

class QuantaIX1CoreTemp(cumulus.sensors.Temp_Unit):
    description = cumulus.sensors.Temp_Unit.intel_cpu_sensor_name
    driver_path = '/sys/devices/platform/coretemp.0'

    temp_fan_min = 65
    temp_max = 90
    temp_crit = 98

class QuantaIX1CY8Temp(cumulus.sensors.Temp_Unit):
    driver_path = ''
    pci_path = '/sys/devices/pci0000:00/0000:00:1f.3'
    if os.path.exists(pci_path):
        for x in os.listdir(pci_path):
            if x.startswith('i2c-'):
                a,b = x.split('-')
                c = b + '-004e'
                driver_path = os.path.join(pci_path, x, c)
                break

    temp_fan_min = 45
    temp_max = 55
    # OTP temp among sensors specified by Quanta is between 60C-70C
    # Critical temp specified is between 53C - 58C
    temp_crit = 65

class QuantaIX1PSU(cumulus.sensors.PSU_Unit):
    def find_and_add_attrs(self):
        if self.get_gpio_value(self.psu_present) == '0':
            self.attrs['psu_pwr1_present'] = 1
        else:
            self.attrs['psu_pwr1_present'] = 0
        if ((self.get_gpio_value(self.psu_ac_ok) == '0') and
            (self.get_gpio_value(self.psu_dc_ok) == '1')):
            self.attrs['psu_pwr1_all_ok'] = 1
        else:
            self.attrs['psu_pwr1_all_ok'] = 0
        return 0

class QuantaIX1LED(cumulus.sensors.StatusLED_Unit):
    good_led_color = '1'  # Green
    bad_led_color = '0'   # Off

    def led_update(self, color):
        #print 'loc: %s color %s' %(self.led_location, color)
        self.set_gpio_value(self.led_location, color)

    def led_read(self):
        color = self.get_gpio_value(self.led_location)
        if color == '0':
            color = 'off'
        else:
            color = 'green'
        return self.name, color

class QuantaIX1PSU1LED(QuantaIX1LED):
    name = 'PSU1'
    led_location = 'psu1_green_led'
    sources = ['PSU1']

class QuantaIX1PSU2LED(QuantaIX1LED):
    name = 'PSU2'
    led_location = 'psu2_green_led'
    sources = ['PSU2']

class QuantaIX1FanLED(QuantaIX1LED):
    name = 'Fans'
    led_location = 'fan_green_led'
    sources = ['Fan1', 'Fan2', 'Fan3', 'Fan4', 'Fan5', 'Fan6', 'Fan7', 'Fan8']

class QuantaIX1PSU1(QuantaIX1PSU):
    name = 'PSU1'
    psu_present = 'psu_pwr1_present'
    psu_ac_ok = 'psu_pwr1_ac_ok'
    psu_dc_ok = 'psu_pwr1_dc_ok'

class QuantaIX1PSU2(QuantaIX1PSU):
    name = 'PSU2'
    psu_present = 'psu_pwr2_present'
    psu_ac_ok = 'psu_pwr2_ac_ok'
    psu_dc_ok = 'psu_pwr2_dc_ok'

class QuantaIX1Fan1(QuantaIX1_Fan):
    fan_present = 'fan1_present'
    name = 'Fan1'
    driver_hwmon = ['fan1']

class QuantaIX1Fan2(QuantaIX1_Fan):
    fan_present = 'fan1_present'
    name = 'Fan2'
    driver_hwmon = ['fan5']

class QuantaIX1Fan3(QuantaIX1_Fan):
    fan_present = 'fan2_present'
    name = 'Fan3'
    driver_hwmon = ['fan2']

class QuantaIX1Fan4(QuantaIX1_Fan):
    fan_present = 'fan2_present'
    name = 'Fan4'
    driver_hwmon = ['fan6']

class QuantaIX1Fan5(QuantaIX1_Fan):
    fan_present = 'fan3_present'
    name = 'Fan5'
    driver_hwmon = ['fan3']

class QuantaIX1Fan6(QuantaIX1_Fan):
    fan_present = 'fan3_present'
    name = 'Fan6'
    driver_hwmon = ['fan7']

class QuantaIX1Fan7(QuantaIX1_Fan):
    fan_present = 'fan4_present'
    name = 'Fan7'
    driver_hwmon = ['fan4']

class QuantaIX1Fan8(QuantaIX1_Fan):
    fan_present = 'fan4_present'
    name = 'Fan8'
    driver_hwmon = ['fan8']

class QuantaIX1Temp1(QuantaIX1CoreTemp):
    name = 'Temp1'
    driver_hwmon = ['temp2']

class QuantaIX1Temp2(QuantaIX1CoreTemp):
    name = 'Temp2'
    driver_hwmon = ['temp3']

class QuantaIX1Temp3(QuantaIX1CoreTemp):
    name = 'Temp3'
    driver_hwmon = ['temp4']

class QuantaIX1Temp4(QuantaIX1CoreTemp):
    name = 'Temp4'
    driver_hwmon = ['temp5']

class QuantaIX1Temp5(QuantaIX1CY8Temp):
    name = 'Temp5'
    driver_hwmon = ['temp1']
    description = 'Rear Outlet Air Temp sensor'

class QuantaIX1Temp6(QuantaIX1CY8Temp):
    name = 'Temp6'
    driver_hwmon = ['temp2']
    description = 'Front Outlet Air Temp sensor'

class QuantaIX1Temp7(QuantaIX1CY8Temp):
    name = 'Temp7'
    driver_hwmon = ['temp3']
    description = 'Front Outlet Air Temp sensor'

class QuantaIX1Temp8(QuantaIX1CY8Temp):
    name = 'Temp8'
    driver_hwmon = ['temp5']
    description = 'CPU Board Temp sensor'

class QuantaIX1Temp9(QuantaIX1CY8Temp):
    name = 'Temp9'
    driver_hwmon = ['temp6']
    description = 'CPU Board Temp sensor'

class QuantaIX1Temp10(cumulus.sensors.Temp_Unit):
    name = 'Temp10'
    description = cumulus.sensors.Temp_Unit.dimm_sensor_name
    driver_hwmon = ['temp1']
    temp_fan_min = 65
    temp_max = 90
    temp_crit = 95

    driver_path = ''
    pci_path = '/sys/devices/pci0000:00/0000:00:1f.3'
    if os.path.exists(pci_path):
        for x in os.listdir(pci_path):
            if x.startswith('i2c-'):
                a,b = x.split('-')
                c = b + '-001a'
                driver_path = os.path.join(pci_path, x, c)
                break

# QuantaIX1, tomahawk internal temp sensors
class QuantaIX1THTemp(cumulus.sensors.Temp_Unit):
    description  = cumulus.sensors.Temp_Unit.mac_die_sensor_name
    driver_path  = '/cumulus/switchd/run'
    temp_max     = 105
    temp_crit    = 110
    temp_fan_min = 67
    temp_fan_max = 82

class QuantaIX1THTemp1(QuantaIX1THTemp):
    name = 'Temp11'
    driver_hwmon = ['temp1']

class QuantaIX1THTemp2(QuantaIX1THTemp):
    name = 'Temp12'
    driver_hwmon = ['temp2']

class QuantaIX1THTemp3(QuantaIX1THTemp):
    name = 'Temp13'
    driver_hwmon = ['temp3']

class QuantaIX1THTemp4(QuantaIX1THTemp):
    name = 'Temp14'
    driver_hwmon = ['temp4']

class QuantaIX1THTemp5(QuantaIX1THTemp):
    name = 'Temp15'
    driver_hwmon = ['temp5']

class QuantaIX1THTemp6(QuantaIX1THTemp):
    name = 'Temp16'
    driver_hwmon = ['temp6']

class QuantaIX1THTemp7(QuantaIX1THTemp):
    name = 'Temp17'
    driver_hwmon = ['temp7']

class QuantaIX1THTemp8(QuantaIX1THTemp):
    name = 'Temp18'
    driver_hwmon = ['temp8']

class QuantaIX1Sensors():
    units = (QuantaIX1Temp1(),
             QuantaIX1Temp2(),
             QuantaIX1Temp3(),
             QuantaIX1Temp4(),
             QuantaIX1Temp5(),
             QuantaIX1Temp6(),
             QuantaIX1Temp7(),
             QuantaIX1Temp8(),
             QuantaIX1Temp9(),
             QuantaIX1Temp10(),
             QuantaIX1THTemp1(),
             QuantaIX1THTemp2(),
             QuantaIX1THTemp3(),
             QuantaIX1THTemp4(),
             QuantaIX1THTemp5(),
             QuantaIX1THTemp6(),
             QuantaIX1THTemp7(),
             QuantaIX1THTemp8(),
             QuantaIX1Fan1(),
             QuantaIX1Fan2(),
             QuantaIX1Fan3(),
             QuantaIX1Fan4(),
             QuantaIX1Fan5(),
             QuantaIX1Fan6(),
             QuantaIX1Fan7(),
             QuantaIX1Fan8(),
             QuantaIX1PSU1(),
             QuantaIX1PSU2(),
             )

class QuantaIX1PWM1(cumulus.sensors.PWM_Unit):
    name = 'System PWM'
    fans = (QuantaIX1Fan1(),
            QuantaIX1Fan2(),
            QuantaIX1Fan3(),
            QuantaIX1Fan4(),
            QuantaIX1Fan5(),
            QuantaIX1Fan6(),
            QuantaIX1Fan7(),
            QuantaIX1Fan8(),
            )
    temps = (QuantaIX1Temp1(),
             QuantaIX1Temp2(),
             QuantaIX1Temp3(),
             QuantaIX1Temp4(),
             QuantaIX1Temp5(),
             QuantaIX1Temp6(),
             QuantaIX1Temp7(),
             QuantaIX1Temp8(),
             QuantaIX1Temp9(),
             QuantaIX1Temp10(),
             QuantaIX1THTemp1(),
             QuantaIX1THTemp2(),
             QuantaIX1THTemp3(),
             QuantaIX1THTemp4(),
             QuantaIX1THTemp5(),
             QuantaIX1THTemp6(),
             QuantaIX1THTemp7(),
             QuantaIX1THTemp8(),
             )

class QuantaIX1PWMs():
    units = (QuantaIX1PWM1(),
             )

class QuantaIX1LEDs():
    units = (QuantaIX1PSU1LED(),
             QuantaIX1PSU2LED(),
             QuantaIX1FanLED(),
             )


class QuantaIX1QSFP28(cumulus.platform.Tomahawk100GPort):
    '''
        Each port has eight bits of LED data controlling four multii-color green
        LEDs. LEDs are set to 10G mode in CPLD in default mode.
        Speed   - Color
        100     - White
         40     - Blue
         25     - Orange
         10     - Green
         50     - Yellow
        The LEDs behave as follows:
          01 - Green on
          11 - both off
    '''
    _set_led_common = '''
;;;
;;; QuantaIX1RangeleyQSFP28
;;;
;;; Two bits per port:
;;;
;;; 1st bit pushed is always zero
;;;
;;; 2n bit pushed on stack controls green LED:
;;; 0 - off
;;; 1 - on
;;;
set_port_led:
        port 1
        pushst PORTSTATUS_ZERO
        pack

        port a

        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor
        push    (BLINK_STATE_P)
        tand
        tinv                    ; flash off with activity
        pushst  PORTSTATUS_LINKEN
        tand
        pack

        ret

set_led_off:
        push    PORTSTATUS_ZERO
        pack
        push    PORTSTATUS_ZERO
        pack

        ret
'''

    _set_led_1x = '''
;;;
;;; In 1x mode, only the green LED of the first bi-color LED is used to
;;; indicate link/activity. The rest are off.
;;;
set_led:
        call    set_port_led
        call    set_led_off
        call    set_led_off
        call    set_led_off

        ret
'''

    _set_led_2x = '''
;;;
;;; In 2x mode, we use the green LED of the first and second bi-color LED to
;;; indicate link/activity. The rest are off.
;;;
set_led:
        call    set_port_led
        inc     a
        inc     a
        call    set_port_led
        call    set_led_off
        call    set_led_off

        ret
'''

    _set_led_4x = '''
;;;
;;; In 4x mode, the green LEDs of all four bi-color LEDs are used to
;;; indicate link/activity.
;;;
set_led:
        call    set_port_led
        inc     a
        call    set_port_led
        inc     a
        call    set_port_led
        inc     a
        call    set_port_led
        ret
'''

    def __init__(self, label, serdes, leds, rx_lane_map, tx_lane_map,
                 rx_polarity_flip, tx_polarity_flip, pre_emphasis=None):
        sfp = cumulus.sfp.SFPROMDTSLabel(label, 'qsfp28')
        cumulus.platform.Tomahawk100GPort.__init__(self,
                                                   label=label,
                                                   serdes=serdes,
                                                   phy_types=None,
                                                   phy_addrs=None,
                                                   phy_layer=sfp.type.upper(),
                                                   leds=leds,
                                                   rx_lane_map=rx_lane_map,
                                                   rx_polarity_flip=rx_polarity_flip,
                                                   tx_lane_map=tx_lane_map,
                                                   tx_polarity_flip=tx_polarity_flip,
                                                   pre_emphasis=pre_emphasis,
                                                   oversub=True,
                                                   unit=0)
        self.sfp = sfp

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        if sub != 0:
            # For breakout modes only update LEDs for the first
            # sub-interface.
            return set_calls

        fn_name = '%s' % self.__class__.__name__
        led_port = self.led_port_num(mode, sub)
        num_lp = self.num_logical_ports(mode)
        if num_lp == 4:
            call = (fn_name + "_4x", self._set_led_4x, led_port)
        elif num_lp == 2:
            call = (fn_name + "_2x", self._set_led_2x, led_port)
        else:
            call = (fn_name + "_1x", self._set_led_1x, led_port)

        set_calls[led_index_list[0]] = call
        set_calls[led_index_list[0]+1] = None
        set_calls[led_index_list[0]+2] = None
        set_calls[led_index_list[0]+3] = None
        set_calls[led_index_list[0]+4] = None
        set_calls[led_index_list[0]+5] = None
        set_calls[led_index_list[0]+6] = None
        set_calls[led_index_list[0]+7] = None

        return set_calls

class QuantaIX1LED(cumulus.ledup.TomahawkLEDController):
    led_functions = (QuantaIX1QSFP28._set_led_common)
    num_leds = 128

class QuantaIX1Switch(cumulus.platform.BCMSwitch):
    ledup = QuantaIX1LED
    led_controllers = set([0, 1])
    _qsfp28 = QuantaIX1QSFP28
    has_phy_ucode_mdio = False
    has_phy_ucode_spi_rom = False

    ports = (
        _qsfp28(label="1", serdes=( 64,  65,  66,  67), leds=(1, 64),
              tx_polarity_flip=0x000a, rx_polarity_flip=0x000b,
              tx_lane_map=0x3012, rx_lane_map=0x1230),
        _qsfp28(label="2", serdes=( 68,  69,  70,  71), leds=(1, 72),
              tx_polarity_flip=0x0002, rx_polarity_flip=0x000f,
              tx_lane_map=0x3120, rx_lane_map=0x2310),
        _qsfp28(label="3", serdes=( 72,  73,  74,  75), leds=(1, 80),
              tx_polarity_flip=0x0002, rx_polarity_flip=0x000e,
              tx_lane_map=0x0213, rx_lane_map=0x2310),
        _qsfp28(label="4", serdes=( 76,  77,  78,  79), leds=(1, 88),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0005,
              tx_lane_map=0x3210, rx_lane_map=0x2130),
        _qsfp28(label="5", serdes=( 44,  45,  46,  47), leds=(1, 24),
              tx_polarity_flip=0x0002, rx_polarity_flip=0x0004,
              tx_lane_map=0x1032, rx_lane_map=0x0231),
        # For A0 boxes
        # _qsfp28(label="6", serdes=( 40,  41,  42,  43), leds=(1, 88),
        #      tx_polarity_flip=0x0002, rx_polarity_flip=0x0004,
        #      tx_lane_map=0x3210, rx_lane_map=0x0312),
        # For B1 boxes
         _qsfp28(label="6", serdes=( 40,  41,  42,  43), leds=(1, 16),
              tx_polarity_flip=0x0002, rx_polarity_flip=0x0004,
              tx_lane_map=0x3210, rx_lane_map=0x0312),
        _qsfp28(label="7", serdes=( 52,  53,  54,  55), leds=(1, 40),
              tx_polarity_flip=0x000d, rx_polarity_flip=0x0009,
              tx_lane_map=0x3120, rx_lane_map=0x0123),
        _qsfp28(label="8", serdes=( 48,  49,  50,  51), leds=(1, 32),
              tx_polarity_flip=0x000d, rx_polarity_flip=0x000a,
              tx_lane_map=0x2103, rx_lane_map=0x3012),
        _qsfp28(label="9", serdes=( 60,  61,  62,  63), leds=(1,  56),
              tx_polarity_flip=0x000e, rx_polarity_flip=0x000a,
              tx_lane_map=0x0123, rx_lane_map=0x1230),
        _qsfp28(label="10", serdes=( 56,  57,  58,  59), leds=(1, 48),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0006,
              tx_lane_map=0x0123, rx_lane_map=0x2310),
        _qsfp28(label="11", serdes=( 36,  37,  38,  39), leds=(1, 8),
              tx_polarity_flip=0x0005, rx_polarity_flip=0x0006,
              tx_lane_map=0x3120, rx_lane_map=0x2103),
        # A0 box
        # _qsfp28(label="12", serdes=( 32,  33,  34,  35), leds=(1, 72),
        #      tx_polarity_flip=0x000f, rx_polarity_flip=0x000a,
        #      tx_lane_map=0x3210, rx_lane_map=0x3021),
        # B1 box
        _qsfp28(label="12", serdes=( 32,  33,  34,  35), leds=(1, 0),
             tx_polarity_flip=0x000f, rx_polarity_flip=0x000a,
             tx_lane_map=0x3210, rx_lane_map=0x3021),
        # A0 box
        # _qsfp28(label="13", serdes=( 80,  81,  82,  83), leds=(1, 96),
        #       tx_polarity_flip=0x0000, rx_polarity_flip=0x0005,
        #      tx_lane_map=0x3012, rx_lane_map=0x3210),
        # B1 box
        _qsfp28(label="13", serdes=( 80,  81,  82,  83), leds=(1, 96),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x000a,
              tx_lane_map=0x1230, rx_lane_map=0x3210),
        _qsfp28(label="14", serdes=( 84,  85,  86,  87), leds=(1, 104),
              tx_polarity_flip=0x0003, rx_polarity_flip=0x0005,
              tx_lane_map=0x1320, rx_lane_map=0x0213),
        # A0 box
        # _qsfp28(label="15", serdes=( 88,  89,  90,  91), leds=(1, 112),
        #     tx_polarity_flip=0x000c, rx_polarity_flip=0x000e,
        #      tx_lane_map=0x2310, rx_lane_map=0x2310),
        # B1 box
        _qsfp28(label="15", serdes=( 88,  89,  90,  91), leds=(1, 112),
              tx_polarity_flip=0x000c, rx_polarity_flip=0x000e,
              tx_lane_map=0x2310, rx_lane_map=0x2310),
        _qsfp28(label="16", serdes=( 92,  93,  94,  95), leds=(1, 120),
              tx_polarity_flip=0x0006, rx_polarity_flip=0x0000,
              tx_lane_map=0x2130, rx_lane_map=0x0321),
        _qsfp28(label="17", serdes=( 96,  97,  98,  99), leds=(0, 64),
              tx_polarity_flip=0x000a, rx_polarity_flip=0x0004,
              tx_lane_map=0x0123, rx_lane_map=0x1302),
        # A0 box
        # _qsfp28(label="18", serdes=(100, 101, 102, 103), leds=(0, 8),
        #     tx_polarity_flip=0x0008, rx_polarity_flip=0x0000,
        #      tx_lane_map=0x2130, rx_lane_map=0x3210),
        # B1 box
        _qsfp28(label="18", serdes=(100, 101, 102, 103), leds=(0, 72),
              tx_polarity_flip=0x0008, rx_polarity_flip=0x0000,
              tx_lane_map=0x2130, rx_lane_map=0x3210),
        _qsfp28(label="19", serdes=(104, 105, 106, 107), leds=(0, 80),
              tx_polarity_flip=0x0002, rx_polarity_flip=0x000d,
              tx_lane_map=0x1302, rx_lane_map=0x1302),
        # A0 box
        # _qsfp28(label="20", serdes=(108, 109, 110, 111), leds=(0, 24),
        #      tx_polarity_flip=0x0000, rx_polarity_flip=0x0008,
        #      tx_lane_map=0x1320, rx_lane_map=0x1230),
        # B1 box
        _qsfp28(label="20", serdes=(108, 109, 110, 111), leds=(0, 88),
             tx_polarity_flip=0x0000, rx_polarity_flip=0x0008,
              tx_lane_map=0x1320, rx_lane_map=0x1230),
        # A0 box
        # _qsfp28(label="21", serdes=( 28,  29,  30,  31), leds=(0, 48),
        #     tx_polarity_flip=0x0002, rx_polarity_flip=0x000f,
        #      tx_lane_map=0x3120, rx_lane_map=0x1032),
        # B1 box
        _qsfp28(label="21", serdes=( 28,  29,  30,  31), leds=(0, 56),
             tx_polarity_flip=0x0002, rx_polarity_flip=0x000f,
              tx_lane_map=0x3120, rx_lane_map=0x1032),
        _qsfp28(label="22", serdes=( 24,  25,  26,  27), leds=(0, 48),
              tx_polarity_flip=0x0003, rx_polarity_flip=0x0005,
              tx_lane_map=0x3012, rx_lane_map=0x0123),
        _qsfp28(label="23", serdes=(   4,   5,   6,   7), leds=(0, 8),
              tx_polarity_flip=0x0005, rx_polarity_flip=0x000c,
              tx_lane_map=0x0213, rx_lane_map=0x3120),
        _qsfp28(label="24", serdes=(   0,   1,   2,   3), leds=(0, 0),
              tx_polarity_flip=0x0007, rx_polarity_flip=0x000a,
              tx_lane_map=0x1230, rx_lane_map=0x3120),
        _qsfp28(label="25", serdes=(  12,  13,  14,  15), leds=(0, 24),
              tx_polarity_flip=0x0005, rx_polarity_flip=0x000f,
              tx_lane_map=0x3201, rx_lane_map=0x2103),
        _qsfp28(label="26", serdes=(   8,   9,  10,  11), leds=(0, 16),
              tx_polarity_flip=0x0006, rx_polarity_flip=0x000b,
              tx_lane_map=0x3120, rx_lane_map=0x2013),
        # A0 box
        # _qsfp28(label="27", serdes=( 20,  21,  22,  23), leds=(0, 32),
        #      tx_polarity_flip=0x0002, rx_polarity_flip=0x0003,
        #      tx_lane_map=0x3120, rx_lane_map=0x2103),
        # B1 box
        _qsfp28(label="27", serdes=( 20,  21,  22,  23), leds=(0, 40),
             tx_polarity_flip=0x0002, rx_polarity_flip=0x0003,
             tx_lane_map=0x3120, rx_lane_map=0x2103),
        _qsfp28(label="28", serdes=( 16,  17,  18,  19), leds=(0, 32),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0007,
              tx_lane_map=0x1302, rx_lane_map=0x1032),
        _qsfp28(label="29", serdes=( 112, 113, 114, 115), leds=(0, 96),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x000c,
              tx_lane_map=0x3120, rx_lane_map=0x3120),
        _qsfp28(label="30", serdes=( 116, 117, 118, 119), leds=(0, 104),
              tx_polarity_flip=0x000f, rx_polarity_flip=0x000a,
              tx_lane_map=0x1032, rx_lane_map=0x0321),
        _qsfp28(label="31", serdes=( 120, 121, 122, 123), leds=(0, 112),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x000f,
              tx_lane_map=0x3210, rx_lane_map=0x0321),
        _qsfp28(label="32", serdes=( 124, 125, 126, 127), leds=(0, 120),
              tx_polarity_flip=0x0003, rx_polarity_flip=0x000a,
              tx_lane_map=0x3021, rx_lane_map=0x0321),

    )

class QuantaIX1(cumulus.platform.Platform):
    name = 'quanta,ix1_rangeley'
    chip = cumulus.platform.TomahawkChip()
    switch = QuantaIX1Switch(chip)
    pwms = QuantaIX1PWMs()
    sensors = QuantaIX1Sensors()
    statusleds = QuantaIX1LEDs()

Platform = QuantaIX1
