#
# Copyright 2016.  Cumulus Networks, Inc.
#
import os
import subprocess
import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors

class QuantaIX2_Fan(cumulus.sensors.Fan_Unit):
    driver_path = '/sys/bus/i2c/devices/110-004e'
    fan_min = 9300
    fan_max = 23000
    rpmmode = 1

    def find_and_add_attrs(self):
        print 'fan_present', self.fan_present
        if self.get_gpio_value(self.fan_present) == '0':
            self.attrs['fan_0'] = 'installed, ok'
        else:
            self.attrs['fan_0'] = 'not_installed'
        return cumulus.sensors.Fan_Unit.find_and_add_attrs(self)

class QuantaIX2CoreTemp(cumulus.sensors.Temp_Unit):
    description = cumulus.sensors.Temp_Unit.intel_cpu_sensor_name
    driver_path = '/sys/devices/platform/coretemp.0'

    temp_fan_min = 65
    temp_max = 90
    temp_crit = 98

class QuantaIX2CY8Temp(cumulus.sensors.Temp_Unit):
    driver_path = '/sys/bus/i2c/devices/110-004e'
    temp_fan_min = 45
    temp_max = 55
    # OTP temp among sensors specified by Quanta is between 60C-70C
    # Critical temp specified is between 53C - 58C
    temp_crit = 65

class QuantaIX2PSU(cumulus.sensors.PSU_Unit):
    def find_and_add_attrs(self):
        if self.get_gpio_value(self.psu_present) == '0':
            self.attrs['psu_pwr1_present'] = 1
        else:
            self.attrs['psu_pwr1_present'] = 0
        if ((self.get_gpio_value(self.psu_ac_ok) == '0') and
            (self.get_gpio_value(self.psu_dc_ok) == '1')):
            self.attrs['psu_pwr1_all_ok'] = 1
        else:
            self.attrs['psu_pwr1_all_ok'] = 0
        return 0

class QuantaIX2LED(cumulus.sensors.StatusLED_Unit):
    good_led_color = '1'  # Green
    bad_led_color = '0'   # Off

    def led_update(self, color):
        #print 'loc: %s color %s' %(self.led_location, color)
        self.set_gpio_value(self.led_location, color)

    def led_read(self):
        color = self.get_gpio_value(self.led_location)
        if color == '0':
            color = 'off'
        else:
            color = 'green'
        return self.name, color

class QuantaIX2PSU1LED(QuantaIX2LED):
    name = 'PSU1'
    led_location = 'psu1_green_led'
    sources = ['PSU1']

class QuantaIX2PSU2LED(QuantaIX2LED):
    name = 'PSU2'
    led_location = 'psu2_green_led'
    sources = ['PSU2']

class QuantaIX2FanLED(QuantaIX2LED):
    name = 'Fans'
    led_location = 'fan_green_led'
    sources = ['Fan1', 'Fan2', 'Fan3', 'Fan4', 'Fan5', 'Fan6', 'Fan7', 'Fan8']

class QuantaIX2PSU1(QuantaIX2PSU):
    name = 'PSU1'
    psu_present = 'psu_pwr1_present'
    psu_ac_ok = 'psu_pwr1_ac_ok'
    psu_dc_ok = 'psu_pwr1_dc_ok'

class QuantaIX2PSU2(QuantaIX2PSU):
    name = 'PSU2'
    psu_present = 'psu_pwr2_present'
    psu_ac_ok = 'psu_pwr2_ac_ok'
    psu_dc_ok = 'psu_pwr2_dc_ok'

class QuantaIX2Fan1(QuantaIX2_Fan):
    fan_present = 'fan1_present'
    name = 'Fan1'
    driver_hwmon = ['fan1']

class QuantaIX2Fan2(QuantaIX2_Fan):
    fan_present = 'fan1_present'
    name = 'Fan2'
    driver_hwmon = ['fan5']

class QuantaIX2Fan3(QuantaIX2_Fan):
    fan_present = 'fan2_present'
    name = 'Fan3'
    driver_hwmon = ['fan2']

class QuantaIX2Fan4(QuantaIX2_Fan):
    fan_present = 'fan2_present'
    name = 'Fan4'
    driver_hwmon = ['fan6']

class QuantaIX2Fan5(QuantaIX2_Fan):
    fan_present = 'fan3_present'
    name = 'Fan5'
    driver_hwmon = ['fan3']

class QuantaIX2Fan6(QuantaIX2_Fan):
    fan_present = 'fan3_present'
    name = 'Fan6'
    driver_hwmon = ['fan7']

class QuantaIX2Fan7(QuantaIX2_Fan):
    fan_present = 'fan4_present'
    name = 'Fan7'
    driver_hwmon = ['fan4']

class QuantaIX2Fan8(QuantaIX2_Fan):
    fan_present = 'fan4_present'
    name = 'Fan8'
    driver_hwmon = ['fan8']

class QuantaIX2Temp1(QuantaIX2CoreTemp):
    name = 'Temp1'
    driver_hwmon = ['temp2']

class QuantaIX2Temp2(QuantaIX2CoreTemp):
    name = 'Temp2'
    driver_hwmon = ['temp3']

class QuantaIX2Temp3(QuantaIX2CoreTemp):
    name = 'Temp3'
    driver_hwmon = ['temp4']

class QuantaIX2Temp4(QuantaIX2CoreTemp):
    name = 'Temp4'
    driver_hwmon = ['temp5']

class QuantaIX2Temp5(QuantaIX2CY8Temp):
    name = 'Temp5'
    driver_hwmon = ['temp1']
    description = 'Rear Outlet Air Temp sensor'

class QuantaIX2Temp6(QuantaIX2CY8Temp):
    name = 'Temp6'
    driver_hwmon = ['temp2']
    description = 'Front Outlet Air Temp sensor'

class QuantaIX2Temp7(QuantaIX2CY8Temp):
    name = 'Temp7'
    driver_hwmon = ['temp3']
    description = 'Front Outlet Air Temp sensor'

class QuantaIX2Temp8(QuantaIX2CY8Temp):
    name = 'Temp8'
    driver_hwmon = ['temp5']
    description = 'CPU Board Temp sensor'

class QuantaIX2Temp9(QuantaIX2CY8Temp):
    name = 'Temp9'
    driver_hwmon = ['temp6']
    description = 'CPU Board Temp sensor'

class QuantaIX2Temp10(cumulus.sensors.Temp_Unit):
    name = 'Temp10'
    description = cumulus.sensors.Temp_Unit.dimm_sensor_name
    driver_path = '/sys/bus/i2c/devices/0-001a'
    driver_hwmon = ['temp1']
    temp_fan_min = 65
    temp_max = 90
    temp_crit = 95


# QuantaIX2, tomahawk internal temp sensors
class QuantaIX2THTemp(cumulus.sensors.Temp_Unit):
    description  = cumulus.sensors.Temp_Unit.mac_die_sensor_name
    driver_path  = '/cumulus/switchd/run'
    temp_max     = 105
    temp_crit    = 110
    temp_fan_min = 65
    temp_fan_max = 90

class QuantaIX2THTemp1(QuantaIX2THTemp):
    name = 'Temp11'
    driver_hwmon = ['temp1']

class QuantaIX2THTemp2(QuantaIX2THTemp):
    name = 'Temp12'
    driver_hwmon = ['temp2']

class QuantaIX2THTemp3(QuantaIX2THTemp):
    name = 'Temp13'
    driver_hwmon = ['temp3']

class QuantaIX2THTemp4(QuantaIX2THTemp):
    name = 'Temp14'
    driver_hwmon = ['temp4']

class QuantaIX2THTemp5(QuantaIX2THTemp):
    name = 'Temp15'
    driver_hwmon = ['temp5']

class QuantaIX2THTemp6(QuantaIX2THTemp):
    name = 'Temp16'
    driver_hwmon = ['temp6']

class QuantaIX2THTemp7(QuantaIX2THTemp):
    name = 'Temp17'
    driver_hwmon = ['temp7']

class QuantaIX2THTemp8(QuantaIX2THTemp):
    name = 'Temp18'
    driver_hwmon = ['temp8']

class QuantaIX2Sensors():
    units = (QuantaIX2Temp1(),
             QuantaIX2Temp2(),
             QuantaIX2Temp3(),
             QuantaIX2Temp4(),
             QuantaIX2Temp5(),
             QuantaIX2Temp6(),
             QuantaIX2Temp7(),
             QuantaIX2Temp8(),
             QuantaIX2Temp9(),
             QuantaIX2Temp10(),
             QuantaIX2THTemp1(),
             QuantaIX2THTemp2(),
             QuantaIX2THTemp3(),
             QuantaIX2THTemp4(),
             QuantaIX2THTemp5(),
             QuantaIX2THTemp6(),
             QuantaIX2THTemp7(),
             QuantaIX2THTemp8(),
             QuantaIX2Fan1(),
             QuantaIX2Fan2(),
             QuantaIX2Fan3(),
             QuantaIX2Fan4(),
             QuantaIX2Fan5(),
             QuantaIX2Fan6(),
             QuantaIX2Fan7(),
             QuantaIX2Fan8(),
             QuantaIX2PSU1(),
             QuantaIX2PSU2(),
             )

class QuantaIX2PWM1(cumulus.sensors.PWM_Unit):
    name = 'System PWM'
    fans = (QuantaIX2Fan1(),
            QuantaIX2Fan2(),
            QuantaIX2Fan3(),
            QuantaIX2Fan4(),
            QuantaIX2Fan5(),
            QuantaIX2Fan6(),
            QuantaIX2Fan7(),
            QuantaIX2Fan8(),
            )
    temps = (QuantaIX2Temp1(),
             QuantaIX2Temp2(),
             QuantaIX2Temp3(),
             QuantaIX2Temp4(),
             QuantaIX2Temp5(),
             QuantaIX2Temp6(),
             QuantaIX2Temp7(),
             QuantaIX2Temp8(),
             QuantaIX2Temp9(),
             QuantaIX2Temp10(),
             QuantaIX2THTemp1(),
             QuantaIX2THTemp2(),
             QuantaIX2THTemp3(),
             QuantaIX2THTemp4(),
             QuantaIX2THTemp5(),
             QuantaIX2THTemp6(),
             QuantaIX2THTemp7(),
             QuantaIX2THTemp8(),
             )

class QuantaIX2PWMs():
    units = (QuantaIX2PWM1(),
             )

class QuantaIX2LEDs():
    units = (QuantaIX2PSU1LED(),
             QuantaIX2PSU2LED(),
             QuantaIX2FanLED(),
             )

class QuantaIX2SFP28(cumulus.platform.Tomahawk25GPort):
    _set_led = '''
;;;
;;; QuantaIX2RangeleySFP28
;;;
;;; Two bits per port:
;;;
;;; 1st bit pushed is always zero
;;;
;;; 2n bit pushed on stack controls green LED:
;;; 0 - off
;;; 1 - on
;;;
set_led:
        push 0
        pack

        ld      b, (ID_ENABLE_P)
        tst     b, ID_ENABLE_BIT
        jc      id_mode

        port a

        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor
        push    (BLINK_STATE_P)
        tand
        tinv                    ; flash off with activity
        pushst  PORTSTATUS_LINKEN
        tand
        pack

        ret
'''
    def __init__(self, label, serdes, leds, rx_lane_map, tx_lane_map,
                 rx_polarity_flip, tx_polarity_flip, pre_emphasis=None):
        sfp = cumulus.sfp.SFPROMDTSLabel(label, 'sfp28')
        cumulus.platform.Tomahawk25GPort.__init__(self,
                                                   label=label,
                                                   serdes=serdes,
                                                   phy_types=None,
                                                   phy_addrs=None,
                                                   phy_layer=sfp.type.upper(),
                                                   leds=leds,
                                                   rx_lane_map=rx_lane_map,
                                                   rx_polarity_flip=rx_polarity_flip,
                                                   tx_lane_map=tx_lane_map,
                                                   tx_polarity_flip=tx_polarity_flip,
                                                   pre_emphasis=pre_emphasis,
                                                   oversub=True,
                                                   unit=0)
        self.sfp = sfp
    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        fn_name = '%s' % self.__class__.__name__
        led_port = self.led_port_num(mode, sub)
        call = (fn_name, self._set_led, led_port)

        set_calls[led_index_list[0]] = call
        set_calls[led_index_list[0] + 1] = None

        return set_calls

class QuantaIX2QSFP28(cumulus.platform.Tomahawk100GPort):
    '''
        Each port has eight bits of LED data controlling four multii-color green
        LEDs. LEDs are set to 10G mode in CPLD in default mode.
        Speed   - Color
        100     - White
         40     - Blue
         25     - Orange
         10     - Green
         50     - Yellow
        The LEDs behave as follows:
          01 - Green on
          11 - both off
    '''
    _set_led_common = '''
;;;
;;; QuantaIX2RangeleyQSFP28
;;;
;;; Two bits per port:
;;;
;;; 1st bit pushed is always zero
;;;
;;; 2n bit pushed on stack controls green LED:
;;; 0 - off
;;; 1 - on
;;;
set_port_led:
        push 0
        pack

        ld      b, (ID_ENABLE_P)
        tst     b, ID_ENABLE_BIT
        jc      id_mode

        port a

        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor
        push    (BLINK_STATE_P)
        tand
        tinv                    ; flash off with activity
        pushst  PORTSTATUS_LINKEN
        tand
        pack

        ret

set_led_off:
        push    0
        pack
        push    0
        pack

        ret

id_mode:
        push    0
        cmp     a, (ID_PORT_P)
        jnz     pack_it
        ld      b, (WIGGLE_STATE_P)
        and     b, (WAG_STATE_P)
        push    b
pack_it:
        pack
        ret
'''

    _set_led_1x = '''
;;;
;;; In 1x mode, only the green LED of the first bi-color LED is used to
;;; indicate link/activity. The rest are off.
;;;
set_led:
        call    set_port_led
        call    set_led_off
        call    set_led_off
        call    set_led_off

        ret
'''

    _set_led_2x = '''
;;;
;;; In 2x mode, we use the green LED of the first and second bi-color LED to
;;; indicate link/activity. The rest are off.
;;;
set_led:
        call    set_port_led
        inc     a
        inc     a
        call    set_port_led
        call    set_led_off
        call    set_led_off

        ret
'''

    _set_led_4x = '''
;;;
;;; In 4x mode, the green LEDs of all four bi-color LEDs are used to
;;; indicate link/activity.
;;;
set_led:
        call    set_port_led
        inc     a
        call    set_port_led
        inc     a
        call    set_port_led
        inc     a
        call    set_port_led
        ret
'''

    def __init__(self, label, serdes, leds, rx_lane_map, tx_lane_map,
                 rx_polarity_flip, tx_polarity_flip, pre_emphasis=None):
        sfp = cumulus.sfp.SFPROMDTSLabel(label, 'qsfp28')
        cumulus.platform.Tomahawk100GPort.__init__(self,
                                                   label=label,
                                                   serdes=serdes,
                                                   phy_types=None,
                                                   phy_addrs=None,
                                                   phy_layer=sfp.type.upper(),
                                                   leds=leds,
                                                   rx_lane_map=rx_lane_map,
                                                   rx_polarity_flip=rx_polarity_flip,
                                                   tx_lane_map=tx_lane_map,
                                                   tx_polarity_flip=tx_polarity_flip,
                                                   pre_emphasis=pre_emphasis,
                                                   oversub=True,
                                                   unit=0)
        self.sfp = sfp

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        if sub != 0:
            # For breakout modes only update LEDs for the first
            # sub-interface.
            return set_calls

        fn_name = '%s' % self.__class__.__name__
        led_port = self.led_port_num(mode, sub)
        num_lp = self.num_logical_ports(mode)
        if num_lp == 4:
            call = (fn_name + "_4x", self._set_led_4x, led_port)
        elif num_lp == 2:
            call = (fn_name + "_2x", self._set_led_2x, led_port)
        else:
            call = (fn_name + "_1x", self._set_led_1x, led_port)

        set_calls[led_index_list[0]] = call
        set_calls[led_index_list[0]+1] = None
        set_calls[led_index_list[0]+2] = None
        set_calls[led_index_list[0]+3] = None
        set_calls[led_index_list[0]+4] = None
        set_calls[led_index_list[0]+5] = None
        set_calls[led_index_list[0]+6] = None
        set_calls[led_index_list[0]+7] = None

        return set_calls

class QuantaIX2LED(cumulus.ledup.TomahawkLEDController):
    led_functions = (QuantaIX2QSFP28._set_led_common)
    num_leds = 128
    has_phys_id = True


class QuantaIX2Switch(cumulus.platform.BCMSwitch):
    ledup = QuantaIX2LED
    led_controllers = set([0, 1])
    _sfp28 = QuantaIX2SFP28
    _qsfp28 = QuantaIX2QSFP28
    has_phy_ucode_mdio = False
    has_phy_ucode_spi_rom = False

    ports = (
  _sfp28(label="1",  serdes=(52,), leds=(1, 40), tx_lane_map=0x0123,
              rx_lane_map=0x0123, tx_polarity_flip=True, rx_polarity_flip=True,),
        _sfp28(label="2",  serdes=(53,), leds=(1, 42), tx_lane_map=0x0123,
              rx_lane_map=0x0123, tx_polarity_flip=True, rx_polarity_flip=False,),
        _sfp28(label="3",  serdes=(54,), leds=(1, 44), tx_lane_map=0x0123,
              rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp28(label="4",  serdes=(55,), leds=(1, 46), tx_lane_map=0x0123,
              rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=True,),
        _sfp28(label="5",  serdes=(72,), leds=(1, 80), tx_lane_map=0x0213,
              rx_lane_map=0x0213, tx_polarity_flip=True, rx_polarity_flip=True,),
        _sfp28(label="6",  serdes=(73,), leds=(1, 82), tx_lane_map=0x0213,
              rx_lane_map=0x0213, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp28(label="7",  serdes=(74,), leds=(1, 84), tx_lane_map=0x0213,
              rx_lane_map=0x0213, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp28(label="8",  serdes=(75,), leds=(1, 86), tx_lane_map=0x0213,
              rx_lane_map=0x0213, tx_polarity_flip=True, rx_polarity_flip=True,),
        _sfp28(label="9",  serdes=(37,), leds=(1, 10), tx_lane_map=0x2310,
              rx_lane_map=0x2310, tx_polarity_flip=False, rx_polarity_flip=True,),
        _sfp28(label="10",  serdes=(40,), leds=(1, 16), tx_lane_map=0x2301,
              rx_lane_map=0x2301, tx_polarity_flip=True, rx_polarity_flip=False,),
        _sfp28(label="11",  serdes=(41,), leds=(1, 18), tx_lane_map=0x2301,
              rx_lane_map=0x2301, tx_polarity_flip=True, rx_polarity_flip=False,),
        _sfp28(label="12",  serdes=(42,), leds=(1, 20), tx_lane_map=0x2301,
              rx_lane_map=0x2301, tx_polarity_flip=True, rx_polarity_flip=False,),
        _sfp28(label="13",  serdes=(43,), leds=(1, 22), tx_lane_map=0x2301,
              rx_lane_map=0x2301, tx_polarity_flip=True, rx_polarity_flip=False,),
        _sfp28(label="14",  serdes=(44,), leds=(1, 24), tx_lane_map=0x1032,
              rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True,),
        _sfp28(label="15",  serdes=(45,), leds=(1, 26), tx_lane_map=0x1032,
              rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True,),
        _sfp28(label="16",  serdes=(46,), leds=(1, 28), tx_lane_map=0x1032,
              rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True,),
        _sfp28(label="17",  serdes=(47,), leds=(1, 30), tx_lane_map=0x1032,
              rx_lane_map=0x1032, tx_polarity_flip=True, rx_polarity_flip=True,),
        _sfp28(label="18",  serdes=(36,), leds=(1, 8), tx_lane_map=0x2310,
              rx_lane_map=0x2310, tx_polarity_flip=True, rx_polarity_flip=False,),
        _sfp28(label="19",  serdes=(38,), leds=(1, 12), tx_lane_map=0x2310,
              rx_lane_map=0x2310, tx_polarity_flip=True, rx_polarity_flip=True,),
        _sfp28(label="20",  serdes=(39,), leds=(1, 14), tx_lane_map=0x2310,
              rx_lane_map=0x2310, tx_polarity_flip=True, rx_polarity_flip=False,),
        _sfp28(label="21",  serdes=(48,), leds=(1, 32), tx_lane_map=0x0213,
              rx_lane_map=0x0213, tx_polarity_flip=True, rx_polarity_flip=True,),
        _sfp28(label="22",  serdes=(49,), leds=(1, 34), tx_lane_map=0x0213,
              rx_lane_map=0x0213, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp28(label="23",  serdes=(50,), leds=(1, 36), tx_lane_map=0x0213,
              rx_lane_map=0x0213, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp28(label="24",  serdes=(51,), leds=(1, 38), tx_lane_map=0x0213,
              rx_lane_map=0x0213, tx_polarity_flip=False, rx_polarity_flip=True,),
        _sfp28(label="25",  serdes=(4,), leds=(0, 8), tx_lane_map=0x3120,
              rx_lane_map=0x3120, tx_polarity_flip=False, rx_polarity_flip=True,),
        _sfp28(label="26",  serdes=(5,), leds=(0, 10), tx_lane_map=0x3120,
              rx_lane_map=0x3120, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp28(label="27",  serdes=(6,), leds=(0, 12), tx_lane_map=0x3120,
              rx_lane_map=0x3120, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp28(label="28",  serdes=(7,), leds=(0, 14), tx_lane_map=0x3120,
              rx_lane_map=0x3120, tx_polarity_flip=True, rx_polarity_flip=True,),
        _sfp28(label="29",  serdes=(8,), leds=(0, 16), tx_lane_map=0x3120,
              rx_lane_map=0x3120, tx_polarity_flip=True, rx_polarity_flip=True,),
        _sfp28(label="30",  serdes=(9,), leds=(0, 18), tx_lane_map=0x3120,
              rx_lane_map=0x3120, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp28(label="31",  serdes=(10,), leds=(0, 20), tx_lane_map=0x3120,
              rx_lane_map=0x3120, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp28(label="32",  serdes=(11,), leds=(0, 22), tx_lane_map=0x3120,
              rx_lane_map=0x3120, tx_polarity_flip=True, rx_polarity_flip=False,),
        _sfp28(label="33",  serdes=(12,), leds=(0, 24), tx_lane_map=0x3120,
              rx_lane_map=0x3120, tx_polarity_flip=True, rx_polarity_flip=True,),
        _sfp28(label="34",  serdes=(13,), leds=(0, 26), tx_lane_map=0x3120,
              rx_lane_map=0x3120, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp28(label="35",  serdes=(14,), leds=(0, 28), tx_lane_map=0x3120,
              rx_lane_map=0x3120, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp28(label="36",  serdes=(15,), leds=(0, 30), tx_lane_map=0x3120,
              rx_lane_map=0x3120, tx_polarity_flip=True, rx_polarity_flip=True,),
        _sfp28(label="37",  serdes=(16,), leds=(0, 32), tx_lane_map=0x2301,
              rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=True,),
        _sfp28(label="38",  serdes=(17,), leds=(0, 34), tx_lane_map=0x2301,
              rx_lane_map=0x2301, tx_polarity_flip=True, rx_polarity_flip=True,),
        _sfp28(label="39",  serdes=(18,), leds=(0, 36), tx_lane_map=0x2301,
              rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp28(label="40",  serdes=(19,), leds=(0, 38), tx_lane_map=0x2301,
              rx_lane_map=0x2301, tx_polarity_flip=True, rx_polarity_flip=True,),
        _sfp28(label="41",  serdes=(112,), leds=(0, 96), tx_lane_map=0x0123,
              rx_lane_map=0x0123, tx_polarity_flip=True, rx_polarity_flip=True,),
        _sfp28(label="42",  serdes=(113,), leds=(0, 98), tx_lane_map=0x0123,
              rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp28(label="43",  serdes=(114,), leds=(0, 100), tx_lane_map=0x0123,
              rx_lane_map=0x0123, tx_polarity_flip=True, rx_polarity_flip=True,),
        _sfp28(label="44",  serdes=(115,), leds=(0, 102), tx_lane_map=0x0123,
              rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp28(label="45",  serdes=(0,), leds=(0, 0), tx_lane_map=0x3210,
              rx_lane_map=0x3210, tx_polarity_flip=True, rx_polarity_flip=True,),
        _sfp28(label="46",  serdes=(1,), leds=(0, 2), tx_lane_map=0x3210,
              rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp28(label="47",  serdes=(2,), leds=(0, 4), tx_lane_map=0x3210,
              rx_lane_map=0x3210, tx_polarity_flip=True, rx_polarity_flip=False,),
        _sfp28(label="48",  serdes=(3,), leds=(0, 6), tx_lane_map=0x3210,
              rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=True,),

        _qsfp28(label="49", serdes=( 68,  69,  70,  71), leds=(1, 72),
              tx_polarity_flip=0x0001, rx_polarity_flip=0x0004,
              tx_lane_map=0x0231, rx_lane_map=0x0231),
        _qsfp28(label="50", serdes=( 76,  77,  78,  79), leds=(1, 88),
              tx_polarity_flip=0x0002, rx_polarity_flip=0x0004,
              tx_lane_map=0x3210, rx_lane_map=0x3210),
        _qsfp28(label="51", serdes=( 80,  81,  82,  83), leds=(1, 96),
              tx_polarity_flip=0x000e, rx_polarity_flip=0x000e,
              tx_lane_map=0x0213, rx_lane_map=0x0213),
        _qsfp28(label="52", serdes=( 84,  85,  86,  87), leds=(1, 104),
              tx_polarity_flip=0x000d, rx_polarity_flip=0x0000,
              tx_lane_map=0x0312, rx_lane_map=0x0312),
        _qsfp28(label="53", serdes=( 96,  97,  98,  99), leds=(0, 64),
              tx_polarity_flip=0x0005, rx_polarity_flip=0x0005,
              tx_lane_map=0x1203, rx_lane_map=0x1203),
         _qsfp28(label="54", serdes=( 100, 101, 102, 103), leds=(0, 72),
              tx_polarity_flip=0x0001, rx_polarity_flip=0x000b,
              tx_lane_map=0x1230, rx_lane_map=0x1230),
        _qsfp28(label="55", serdes=( 104, 105, 106, 107), leds=(0, 80),
              tx_polarity_flip=0x000a, rx_polarity_flip=0x0005,
              tx_lane_map=0x3012, rx_lane_map=0x3012),
        _qsfp28(label="56", serdes=( 108, 109,  110, 111), leds=(0, 88),
              tx_polarity_flip=0x0007, rx_polarity_flip=0x000c,
              tx_lane_map=0x1230, rx_lane_map=0x1230),

    )

class QuantaIX2(cumulus.platform.Platform):
    name = 'quanta,ix2_rangeley'
    chip = cumulus.platform.TomahawkChip()
    switch = QuantaIX2Switch(chip)
    pwms = QuantaIX2PWMs()
    sensors = QuantaIX2Sensors()
    statusleds = QuantaIX2LEDs()

Platform = QuantaIX2
