#
# Copyright 2017.  Cumulus Networks, Inc.
#

import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors
import subprocess
import re
import os

''' Quanta LY4R
'''

vhwmon_path = '/sys/class/misc/vhwmon/bmc:'

class QuantaLY4RCoreTemp(cumulus.sensors.Temp_Unit):
    description = cumulus.sensors.Temp_Unit.intel_cpu_sensor_name
    driver_path = '/sys/devices/platform/coretemp.0'
    temp_max = 90
    temp_crit = 95
    no_shutdown = True

class QuantaLY4RTemp1(QuantaLY4RCoreTemp):
    name = 'Temp1'
    description = 'Core 0'
    driver_hwmon = ['temp2']

class QuantaLY4RTemp2(QuantaLY4RCoreTemp):
    name = 'Temp2'
    description = 'Core 1'
    driver_hwmon = ['temp3']

class QuantaLY4RSysTemp(cumulus.sensors.Temp_Unit):
    driver_path = vhwmon_path + 'temp'

class QuantaLY4RTemp3(QuantaLY4RSysTemp):
    name = 'Temp3'
    description = 'Ambient Temp 0'
    temp_crit = 52
    temp_max = 50
    driver_hwmon = ['temp1']

class QuantaLY4RTemp4(QuantaLY4RSysTemp):
    name = 'Temp4'
    description = 'Ambient Temp 1'
    temp_crit = 82
    temp_max = 80
    driver_hwmon = ['temp2']

class QuantaLY4RTemp5(QuantaLY4RSysTemp):
    name = 'Temp5'
    description = 'Ambient Temp 2'
    temp_crit = 82
    temp_max = 80
    driver_hwmon = ['temp3']

class QuantaLY4RTemp6(QuantaLY4RSysTemp):
    name = 'Temp6'
    description = 'Ambient Temp 3'
    temp_crit = 88
    temp_max = 85
    driver_hwmon = ['temp4']

class QuantaLY4RTemp7(QuantaLY4RSysTemp):
    name = 'Temp7'
    description = 'Ambient Temp 4'
    temp_crit = 72
    temp_max = 70
    driver_hwmon = ['temp5']

class QuantaLY4RTemp8(QuantaLY4RSysTemp):
    name = 'Temp8'
    description = 'CPU Temp'
    temp_crit = 97
    temp_max = 96
    driver_hwmon = ['temp6']

class QuantaLY4R_PSU(cumulus.sensors.PSU_Unit):
    def get_status(self):
        val = self.attrs.get('power1_input', 0)
        if val != 0:
            val = 1
        self.attrs['psu_pwr_all_ok'] = val
        return cumulus.sensors.PSU_Unit.get_status(self)

class QuantaLY4R_PSU1(QuantaLY4R_PSU):
    name = 'PSU1'
    driver_hwmon = ['power1']

class QuantaLY4R_PSU1_v2(QuantaLY4R_PSU1):
    driver_path = vhwmon_path + 'psu1'

class QuantaLY4R_PSU1_v1(QuantaLY4R_PSU1):
    '''
    On the single PSU box version, the only PSU is accessed
    as psu2.
    '''
    driver_path = vhwmon_path + 'psu2'

class QuantaLY4R_PSU2(QuantaLY4R_PSU):
    name = 'PSU2'
    driver_path = vhwmon_path + 'psu2'
    driver_hwmon = ['power1']

class QuantaLY4RTemp9(cumulus.sensors.Temp_Unit):
    name = 'PSU1Temp1'
    description = 'PSU1 Temp1'
    driver_path = vhwmon_path + 'psu1'
    driver_hwmon = ['temp1']
    temp_crit = 100
    temp_max = 95

class QuantaLY4RTemp9_v1(QuantaLY4RTemp9):
    driver_path = vhwmon_path + 'psu2'

class QuantaLY4RTemp9_v2(QuantaLY4RTemp9):
    driver_path = vhwmon_path + 'psu1'

class QuantaLY4RTemp10(cumulus.sensors.Temp_Unit):
    name = 'PSU1Temp2'
    description = 'PSU1 Temp2'
    driver_hwmon = ['temp2']
    temp_crit = 110
    temp_max = 105

class QuantaLY4RTemp10_v1(QuantaLY4RTemp10):
    driver_path = vhwmon_path + 'psu2'

class QuantaLY4RTemp10_v2(QuantaLY4RTemp10):
    driver_path = vhwmon_path + 'psu1'

class QuantaLY4RTemp11(cumulus.sensors.Temp_Unit):
    name = 'PSU2Temp1'
    description = 'PSU2 Temp1'
    driver_path = vhwmon_path + 'psu2'
    driver_hwmon = ['temp1']
    temp_crit = 100
    temp_max = 95

class QuantaLY4RTemp12(cumulus.sensors.Temp_Unit):
    name = 'PSU2Temp2'
    description = 'PSU2 Temp2'
    driver_path = vhwmon_path + 'psu2'
    driver_hwmon = ['temp2']
    temp_crit = 110
    temp_max = 105

class QuantaLY4RFan(cumulus.sensors.Fan_Unit):
    fan_min = 500
    fan_max = 9000
    driver_path = vhwmon_path + 'fan'
    no_pwm = True

class QuantaLY4RFan1(QuantaLY4RFan):
    name = 'Fan1'
    description = 'Fan 1'
    driver_hwmon = ['fan1']

class QuantaLY4RFan2(QuantaLY4RFan):
    name = 'Fan2'
    description = 'Fan 2'
    driver_hwmon = ['fan2']

class QuantaLY4RFan3(QuantaLY4RFan):
    name = 'Fan3'
    description = 'Fan 3'
    driver_hwmon = ['fan3']

class QuantaLY4RSystemLED(cumulus.sensors.StatusLED_Unit):
    '''
        the system LED is controlled via two GPIO pins, led_status, led_boot:

                        status
                  0             1
        boot   +---------|-----------+
         0     |  off    |    green  |
               +---------|-----------+
         1     |  amber  |    off    |
               +---------+-----------+
    '''
    name = 'System'
    status_colormap = {'green':'1',  'yellow':'0',  'off':'0'}
    boot_colormap   = {'green':'0',  'yellow':'1',  'off':'0'}
    colormap        = {'10':'green', '11':'off', '01':'yellow', '00':'off'}

    def led_update(self, color):
        self.set_gpio_value('led_status', self.status_colormap[color])
        self.set_gpio_value('led_boot',   self.boot_colormap[color])

    def led_read(self):
        status_value = self.get_gpio_value('led_status')
        boot_value = self.get_gpio_value('led_boot')
        return self.name, self.colormap[status_value + boot_value]

def quantaLY4R_num_psus():
    num_psus = 2
    if os.geteuid() == 0:
        try:
            output = subprocess.check_output(['/usr/bin/ipmitool', 'fru', 'print'])
            match = re.search('Chassis Extra\s*: 0\d0(\d)0\d', output)
            if match:
                if match.group(1) == '1':
                    num_psus = 1
        except:
            pass
    return num_psus

class QuantaLY4RSensors():
    units = (
        QuantaLY4RTemp1(),
        QuantaLY4RTemp2(),
        QuantaLY4RTemp3(),
        QuantaLY4RTemp4(),
        QuantaLY4RTemp5(),
        QuantaLY4RTemp6(),
        QuantaLY4RTemp7(),
        QuantaLY4RTemp8(),
        QuantaLY4RFan1(),
        QuantaLY4RFan2(),
        QuantaLY4RFan3(),
    )

    if quantaLY4R_num_psus() == 1:
        units += (
            QuantaLY4R_PSU1_v1(),
            QuantaLY4RTemp9_v1(),
            QuantaLY4RTemp10_v1(),
        )
    else:
        units += (
            QuantaLY4R_PSU1_v2(),
            QuantaLY4R_PSU2(),
            QuantaLY4RTemp9_v2(),
            QuantaLY4RTemp10_v2(),
            QuantaLY4RTemp11(),
            QuantaLY4RTemp12(),
        )

class QuantaLY4RLEDs():
    units = (
        QuantaLY4RSystemLED(),
    )

''' There doesn't seem to by any logic to how ports are mapped to their
    hw port number. This table maps swp to hw_port.
'''
ly4rHwPort_map = [
    -1, 9, 8, 7, 6, 5, 4, 3, 2, 10, 11, 12, 13, 14,
    15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 28
]

class QuantaLY4RPhylessGigE(cumulus.platform.Hurricane21GPort):
    _set_led = '''
;;; QuantaLY4RPhylessGigE

set_led:
        call    set_led_common
        ret
'''
    _set_led_common = '''

set_led_common:
        port a
        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor
        push    (BLINK_STATE_P)
        tand
        tinv                    ; flash off with activity
        pushst  PORTSTATUS_LINKEN
        tand
        tinv
        pack

        port a
        pushst PORTSTATUS_ONE
        pack

        port a
        pushst PORTSTATUS_ONE
        pack

        ret
'''

    def __init__(self, label, hw_port, prim_offset, unit, leds):
        cumulus.platform.Hurricane21GPort.__init__(self,
                                                   label=label,
                                                   hw_port=hw_port,
                                                   phy_type=None,
                                                   phy_addr=None,
                                                   phy_layer='1GBaseT',
                                                   prim_offset=prim_offset,
                                                   leds=leds,
                                                   unit=unit)

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)
        set_calls = {}

        fn_name = '%s' % self.__class__.__name__
        set_calls[led_index_list[0]] = (fn_name, self._set_led, ly4rHwPort_map[led_port])
        set_calls[led_index_list[0] + 1] = None
        set_calls[led_index_list[0] + 2] = None
        return set_calls


class QuantaLY4RPhyGigE(cumulus.platform.Hurricane21GPort):
    _set_led = '''
;;; QuantaLY4RPhyGigE
set_led:
        call    set_led_common
        ret
'''

    def __init__(self, label, hw_port, phy_addr, prim_offset, unit, leds):
        cumulus.platform.Hurricane21GPort.__init__(self,
                                                   label=label,
                                                   hw_port=hw_port,
                                                   phy_type="54282",
                                                   phy_addr=phy_addr,
                                                   phy_layer='1GBaseT',
                                                   prim_offset=prim_offset,
                                                   leds=leds,
                                                   unit=unit)

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)
        set_calls = {}

        fn_name = '%s' % self.__class__.__name__
        set_calls[led_index_list[0]] = (fn_name, self._set_led, ly4rHwPort_map[led_port])
        set_calls[led_index_list[0] + 1] = None
        set_calls[led_index_list[0] + 2] = None
        return set_calls


class QuantaLY4RSFPp(cumulus.platform.Hurricane210GPort):
    _set_led = '''
;;; QuantaLY4RSFPp
set_led:
        call    set_led_common
        ret
'''

    def __init__(self, label, hw_port, leds, unit):
        cumulus.platform.Hurricane210GPort.__init__(self,
                                                    label=label,
                                                    hw_port=hw_port,
                                                    phy_type=None,
                                                    phy_addr=None,
                                                    phy_layer='SFP+',
                                                    leds=leds,
                                                    unit=unit)
        self.sfp = cumulus.sfp.SFPROMDTSLabel(label, 'sfp+')

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)
        set_calls = {}

        fn_name = '%s' % self.__class__.__name__
        set_calls[led_index_list[0]] = (fn_name, self._set_led, ly4rHwPort_map[led_port])
        set_calls[led_index_list[0] + 1] = None
        set_calls[led_index_list[0] + 2] = None
        return set_calls

class QuantaLY4RLED(cumulus.ledup.Hurricane2LEDController):
    led_functions = (QuantaLY4RPhylessGigE._set_led_common)
    num_leds = 78

class QuantaLY4RBMC(cumulus.platform.BMC):
    name = 'bmc'
    type = 'ipmibmc'

class QuantaLY4RSwitch(cumulus.platform.BCMSwitch):
    ledup = QuantaLY4RLED
    _phyless_gige = QuantaLY4RPhylessGigE
    _phy_gige = QuantaLY4RPhyGigE
    _sfp = QuantaLY4RSFPp
    _fab = cumulus.platform.Hurricane213GFabricPort
    has_phy_ucode_mdio = False
    has_phy_ucode_spi_rom = False

    ports = (
        _phyless_gige(label="1", hw_port='ge1', prim_offset=0x0800, unit=0, leds=(0, 0)),
        _phyless_gige(label="2", hw_port='ge0', prim_offset=0x0801, unit=0, leds=(0, 3)),
        _phyless_gige(label="3", hw_port='ge3', prim_offset=0x0803, unit=0, leds=(0, 6)),
        _phyless_gige(label="4", hw_port='ge2', prim_offset=0x0802, unit=0, leds=(0, 9)),
        _phyless_gige(label="5", hw_port='ge5', prim_offset=0x0805, unit=0, leds=(0, 12)),
        _phyless_gige(label="6", hw_port='ge4', prim_offset=0x0804, unit=0, leds=(0, 15)),
        _phyless_gige(label="7", hw_port='ge7', prim_offset=0x0807, unit=0, leds=(0, 18)),
        _phyless_gige(label="8", hw_port='ge6', prim_offset=0x0806, unit=0, leds=(0, 21)),
        _phyless_gige(label="9", hw_port='ge9', prim_offset=0x1001, unit=0, leds=(0, 24)),
        _phyless_gige(label="10", hw_port='ge8', prim_offset=0x1000, unit=0, leds=(0, 27)),
        _phyless_gige(label="11", hw_port='ge11', prim_offset=0x1003, unit=0, leds=(0, 30)),
        _phyless_gige(label="12", hw_port='ge10', prim_offset=0x1002, unit=0, leds=(0, 33)),
        _phyless_gige(label="13", hw_port='ge13', prim_offset=0x1005, unit=0, leds=(0, 36)),
        _phyless_gige(label="14", hw_port='ge12', prim_offset=0x1004, unit=0, leds=(0, 39)),
        _phyless_gige(label="15", hw_port='ge15', prim_offset=0x1007, unit=0, leds=(0, 42)),
        _phyless_gige(label="16", hw_port='ge14', prim_offset=0x1006, unit=0, leds=(0, 45)),

        _phy_gige(label="17", hw_port='ge17', prim_offset=0x0001, unit=0, leds=(0, 48), phy_addr=0x1),
        _phy_gige(label="18", hw_port='ge16', prim_offset=0x0000, unit=0, leds=(0, 51), phy_addr=0x0),
        _phy_gige(label="19", hw_port='ge19', prim_offset=0x0003, unit=0, leds=(0, 54), phy_addr=0x3),
        _phy_gige(label="20", hw_port='ge18', prim_offset=0x0002, unit=0, leds=(0, 57), phy_addr=0x2),
        _phy_gige(label="21", hw_port='ge21', prim_offset=0x0005, unit=0, leds=(0, 60), phy_addr=0x5),
        _phy_gige(label="22", hw_port='ge20', prim_offset=0x0004, unit=0, leds=(0, 63), phy_addr=0x4),
        _phy_gige(label="23", hw_port='ge23', prim_offset=0x0007, unit=0, leds=(0, 66), phy_addr=0x7),
        _phy_gige(label="24", hw_port='ge22', prim_offset=0x0006, unit=0, leds=(0, 69), phy_addr=0x6),

        _phyless_gige(label="25", hw_port='ge1', prim_offset=0x0000, unit=1, leds=(1, 0)),
        _phyless_gige(label="26", hw_port='ge0', prim_offset=0x0001, unit=1, leds=(1, 3)),
        _phyless_gige(label="27", hw_port='ge3', prim_offset=0x0003, unit=1, leds=(1, 6)),
        _phyless_gige(label="28", hw_port='ge2', prim_offset=0x0002, unit=1, leds=(1, 9)),
        _phyless_gige(label="29", hw_port='ge5', prim_offset=0x0005, unit=1, leds=(1, 12)),
        _phyless_gige(label="30", hw_port='ge4', prim_offset=0x0004, unit=1, leds=(1, 15)),
        _phyless_gige(label="31", hw_port='ge7', prim_offset=0x0007, unit=1, leds=(1, 18)),
        _phyless_gige(label="32", hw_port='ge6', prim_offset=0x0006, unit=1, leds=(1, 21)),
        _phyless_gige(label="33", hw_port='ge9', prim_offset=0x0801, unit=1, leds=(1, 24)),
        _phyless_gige(label="34", hw_port='ge8', prim_offset=0x0800, unit=1, leds=(1, 27)),
        _phyless_gige(label="35", hw_port='ge11', prim_offset=0x0803, unit=1, leds=(1, 30)),
        _phyless_gige(label="36", hw_port='ge10', prim_offset=0x0802, unit=1, leds=(1, 33)),
        _phyless_gige(label="37", hw_port='ge13', prim_offset=0x0805, unit=1, leds=(1, 36)),
        _phyless_gige(label="38", hw_port='ge12', prim_offset=0x0804, unit=1, leds=(1, 39)),
        _phyless_gige(label="39", hw_port='ge15', prim_offset=0x0807, unit=1, leds=(1, 42)),
        _phyless_gige(label="40", hw_port='ge14', prim_offset=0x0806, unit=1, leds=(1, 45)),

        _phy_gige(label="41", hw_port='ge17', prim_offset=0x1001, unit=1, leds=(1, 48), phy_addr=0x2),
        _phy_gige(label="42", hw_port='ge16', prim_offset=0x1000, unit=1, leds=(1, 51), phy_addr=0x1),
        _phy_gige(label="43", hw_port='ge19', prim_offset=0x1003, unit=1, leds=(1, 54), phy_addr=0x4),
        _phy_gige(label="44", hw_port='ge18', prim_offset=0x1002, unit=1, leds=(1, 57), phy_addr=0x3),
        _phy_gige(label="45", hw_port='ge21', prim_offset=0x1005, unit=1, leds=(1, 60), phy_addr=0x6),
        _phy_gige(label="46", hw_port='ge20', prim_offset=0x1004, unit=1, leds=(1, 63), phy_addr=0x5),
        _phy_gige(label="47", hw_port='ge23', prim_offset=0x1007, unit=1, leds=(1, 66), phy_addr=0x8),
        _phy_gige(label="48", hw_port='ge22', prim_offset=0x1006, unit=1, leds=(1, 69), phy_addr=0x7),

        _sfp(label="49", hw_port='xe0', leds=(0, 72), unit=0),
        _sfp(label="50", hw_port='xe1', leds=(0, 75), unit=0),
        _sfp(label="51", hw_port='xe0', leds=(1, 72), unit=1),
        _sfp(label="52", hw_port='xe1', leds=(1, 75), unit=1),

        _fab(label="0", hw_port='hg0', unit=0),
        _fab(label="1", hw_port='hg1', unit=0),
        _fab(label="2", hw_port='hg0', unit=1),
        _fab(label="3", hw_port='hg1', unit=1),
    )

class QuantaLY4R(cumulus.platform.Platform):
    name = 'quanta,ly4r'
    chip = cumulus.platform.Hurricane2Chip()
    switch = QuantaLY4RSwitch(chip)
    statusleds = QuantaLY4RLEDs()
    pwms = None
    sensors = QuantaLY4RSensors()
    bmc = QuantaLY4RBMC()

Platform = QuantaLY4R
