# (C) 2012 Kirill Elagin <kirelagin@gmail.com>
#                      http://kir.elagin.me/
# 
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without modification,
# are permitted provided that the following conditions are met:
# 
#         * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
# 
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
# 
#     * The name of the author may not be used to endorse or promote products
#       derived from this software without specific prior written permission.
# 
# 
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR "AS IS" AND ANY EXPRESS OR IMPLIED
# WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
# EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
# OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
# IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
# THE POSSIBILITY OF SUCH DAMAGE.

import errno
import os
import socket


def sd_notifyd(env, unset_environment=False):
    """More pythonic sd_notify that takes a dictionary."""
    state = '\n'.join(['{0}={1}'.format(k,v) for (k,v) in env.iteritems()])
    return sd_notify(unset_environment, state)

def sd_notify(unset_environment, state):
    """Notify service manager about start-up completion and other daemon
    status changes.

    This is reimplementation of systemd's reference sd_notify.

    """
    sock = None

    try:
        if not state:
            return -errno.EINVAL

        e = os.environ.get('NOTIFY_SOCKET', None)
        if not e:
            return 0
        if e[0] not in ('@', '/') or len(e) == 1:
            return -errno.EINVAL

        if e[0] == '@':
            e = '\0' + e[1:]

        # SOCK_CLOEXEC was added in Python 3.2 and requires Linux >= 2.6.27.
        #sock = socket.socket(socket.AF_UNIX, socket.SOCK_DGRAM \
        #       | socket.SOCK_CLOEXEC)
        sock = socket.socket(socket.AF_UNIX, socket.SOCK_DGRAM)

        # sendmsg() was added in Python 3.3,
        # so using sendto() instead.
        # Python seems to lack MSG_NOSIGNAL flag.
        if sock.sendto(state, e) > 0:
            return 1
    except e:
        pass
    finally:
        if sock:
            sock.close()
        if unset_environment:
            if 'NOTIFY_SOCKET' in os.environ:
                del os.environ['NOTIFY_SOCKET']
    return 0
